// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
// more details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
#include "adjustDifficulty.h"
#include "dialogWnd.h"
#include "resource.h"
#include <commctrl.h>
#include "toolbar.h"

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
static BOOL CALLBACK DiffiProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
static void DiffSetPos(HWND hDlg, bool showOnly, int* percents=NULL);

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
void DoAdjustDiff(HWND hParent, UINT id)
{
	if( CanGame() )
	{
		BOOL popup = (BOOL)(id == ID_TYPE_ADJUST);
		CreateDialogParam(GetModuleHandle(NULL), MAKEINTRESOURCE(IDD_DIFFCUSTO), hParent, DiffiProc, (LPARAM)popup);
	}
}

// ----------------------------------------------------------------------------
static BOOL CALLBACK DiffiProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	static bool reset = false;

	switch( uMsg )
	{
	case WM_INITDIALOG:
		{
			gameDlg = hDlg;
			Toolbar_BtnCheck(gameTBar, ID_DIFFICULTY, true);

			reset = true;
			DiffSetPos(hDlg, false);
			reset = false;

			RECT tr = Toolbar_BtnRect(gameTBar, ID_DIFFICULTY, true);

			return InitDlgProc(_T("DifficultyDialog"), hDlg, tr.left, tr.bottom+1);
		}
		break;

	case WM_DESTROY:
		{
			Toolbar_BtnCheck(gameTBar, ID_DIFFICULTY, false);
			SetFocus(GetParent(hDlg));
			gameDlg = NULL;
		}
		break;

	case WM_ACTIVATE:
		{
			if( LOWORD(wParam) == WA_INACTIVE )
			{
				if( GetParent((HWND)lParam) != hDlg )
					DestroyWindow(hDlg);
			}
		}
		break;

	case WM_NOTIFY:
		{
			if( reset ) break;

			bool    gtset =  false;
			bool    gtchg =  false;
			LPNMHDR pNMH  = (LPNMHDR)lParam;

			switch( pNMH->idFrom )
			{
			case IDC_MINIMUM:
			case IDC_BEGINNER:
			case IDC_EASY:
			case IDC_MEDIUM:
			case IDC_HARD:
			case IDC_EXTREME: gtset = true; break;
			}

			if( !gtset ) break;

			reset = true;
			DiffSetPos(hDlg, true);
			reset = false;

			for( int gtchk = gameExtreme; gtchk >= gameBeginner; gtchk-- )
			{
				for( int gt = gtchk-1; gt >= gameMinimum; gt-- )
				{
					if( gameDiff[gt] > gameDiff[gtchk] )
					{
						gameDiff[gt] = gameDiff[gtchk];
						gtchg        = true;
					}
				}
			}

			if( gtchg )
			{
				reset = true;
				DiffSetPos(hDlg, false);
				reset = false;
			}
		}
		break;

	case WM_COMMAND:
		{
			switch( wParam )
			{
			case IDCANCEL:
			case IDOK:
				{
					DestroyWindow(hDlg);
				}
				break;

			case IDC_RESET:
				{
					reset = true;
					int percents[6];
					SetDiffDefaults(percents);
					DiffSetPos(hDlg, false, percents);
					reset = false;
				}
				break;
			}
		}
		break;
	}

	return FALSE;
}

// ----------------------------------------------------------------------------
static void DiffSetPos(HWND hDlg, bool showOnly, int* percents)
{
	UINT sliders[] = { IDC_MINIMUM,    IDC_BEGINNER,    IDC_EASY,    IDC_MEDIUM,    IDC_HARD,    IDC_EXTREME    };
	UINT labels [] = { IDC_MINIMUMPOS, IDC_BEGINNERPOS, IDC_EASYPOS, IDC_MEDIUMPOS, IDC_HARDPOS, IDC_EXTREMEPOS };

	if( !showOnly )
	{
		for( int c = 0; c < SIZEOF(sliders); c++ )
		{
			if( percents ) gameDiff[c] = percents[c];
			SendDlgItemMessage(hDlg, sliders[c], TBM_SETPOS, TRUE, gameDiff[c]);
//			SendDlgItemMessage(hDlg, sliders[c], TBM_SETPOS, FALSE, gameDiff[c]);
		}
	}

	for( int c = 0; c < SIZEOF(labels); c++ )
	{
		gameDiff[c] = SendDlgItemMessage(hDlg, sliders[c], TBM_GETPOS, 0, 0);
		SetDlgItemInt(hDlg, labels[c], gameDiff[c], FALSE);
//		EnableWindow(GetDlgItem(hDlg, sliders[c]), TRUE);
//		SendDlgItemMessage(hDlg, sliders[c], TBM_SETPOS, TRUE, gameDiff[c]);
//		InvalidateRect(GetDlgItem(hDlg, sliders[c]), NULL, TRUE);
	}
}
