// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
// more details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
#include "dialogWnd.h"
#include "shadowWnd.h"
#include "system.h"
#include "locale.h"
#include "wnd.h"

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
static void SetWndLocale(HWND hWnd, LPCTSTR sect)
{
	TCHAR    text[1024];
	GetWindowText(hWnd, text, SIZEOF(text));
	LPCTSTR  textn = LCL_String(sect, text, false);
	SetWindowText(hWnd, textn);
}

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
static void SetLocale(HWND hDlg, LPCTSTR sect)
{
	SetWndLocale(hDlg, sect);

	HWND hChild = GetWindow(hDlg, GW_CHILD);

	while( hChild )
	{
		SetWndLocale(hChild, sect);
		hChild = GetWindow(hChild, GW_HWNDNEXT);
	}
}

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
BOOL InitDlgProc(LPCTSTR sect, HWND hDlg, int dx, int dy, bool rightAlign)
{
	TCHAR wndClass[MAX_PATH];
	WNDCLASSEX wce = { sizeof(wce) };
	GetClassName(GetParent(hDlg), wndClass, MAX_PATH);
	GetClassInfoEx(GetModuleHandle(NULL), wndClass, &wce);
	SendMessage(hDlg, WM_SETICON, ICON_BIG,   (LPARAM)wce.hIcon);
	SendMessage(hDlg, WM_SETICON, ICON_SMALL, (LPARAM)wce.hIconSm);

	SetLocale(hDlg, sect);

	RECT dr = Wnd_OuterRect(hDlg);

	if( dx == 65535 || dy == 65535 )
	{
		RECT wr  = Wnd_OuterRect(GetParent(hDlg));

		dx = dx != 65535 ? dx : wr.left + ((wr.right -wr.left)-(dr.right -dr.left))/2;
		dy = dy != 65535 ? dy : wr.top  + ((wr.bottom-wr.top) -(dr.bottom-dr.top ))/2;
	}

	if( (GetWindowLong(hDlg, GWL_STYLE) & WS_CHILD) == WS_CHILD )
	{
		POINT pt = { dx, dy };
		MapWindowPoints(NULL, GetParent(hDlg), &pt, 1);
		dx = pt.x;
		dy = pt.y;
	}

	if( dx != 65535 && dy != 65535 && rightAlign )
	{
		dx -= dr.right - dr.left;
	}

	SetWindowPos(hDlg, HWND_TOP, dx, dy, 0, 0, SWP_NOSIZE);
	ShowWindow  (hDlg, SW_SHOWNORMAL);

	if( Sys_OSVersion() >= osWinXP ) CreateShadow(hDlg);

	SetFocus(hDlg);

	return TRUE;
}
