// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
// more details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
#include "locale.h"

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
static TCHAR*  gSects  = NULL;
static TCHAR** gKeys   = NULL;
static int     gNSects = 0;

// ----------------------------------------------------------------------------
void LCL_Load(LPCTSTR homeDir)
{
	TCHAR    profile[MAXPATH];
	wsprintf(profile, _T("%s\\locale.ini"), homeDir);

	DWORD    len   = 0;
	DWORD    size  = 32;
	LPCTSTR  sect  = NULL;
	HANDLE   hHeap = GetProcessHeap();
	int      s     = 0;

	LCL_Unload();

	gSects = (TCHAR*)HeapAlloc(hHeap, 0, size*sizeof(TCHAR));

	do
	{
		size  *= 2;
		gSects = (TCHAR*)HeapReAlloc(hHeap, 0, gSects, size*sizeof(TCHAR));
		len    = GetPrivateProfileSectionNames(gSects, size, profile);
	}
	while( len == size-2 );

	sect    = gSects;
	gNSects = 0;

	while( *sect )
	{
		gNSects++;
		sect += lstrlen(sect) + 1;
	}

	gKeys = (TCHAR**)HeapAlloc(hHeap, 0, gNSects*sizeof(TCHAR*));
	sect  = gSects;

	while( *sect )
	{
		size     =  32;
		gKeys[s] = (TCHAR*)HeapAlloc(hHeap, 0, size*sizeof(TCHAR));

		do
		{
			size    *= 2;
			gKeys[s] = (TCHAR*)HeapReAlloc(hHeap, 0,  gKeys[s], size*sizeof(TCHAR));
			len      = GetPrivateProfileSection(sect, gKeys[s], size, profile);
		}
		while( len == size-2 );

		sect += lstrlen(sect) + 1;
		s++;
	}
}

// ----------------------------------------------------------------------------
void LCL_Unload()
{
	if( gSects )
	{
		HeapFree(GetProcessHeap(), 0, (void*)gSects);
		gSects = NULL;
	}

	if( gKeys )
	{
		for( int s = 0; s < gNSects; s++ )
		{
			HeapFree(GetProcessHeap(), 0, (void*)gKeys[s]);
			gKeys[s] = NULL;
		}

		HeapFree(GetProcessHeap(), 0, (void*)gKeys);
		gKeys = NULL;
	}
}

// ----------------------------------------------------------------------------
static LPCTSTR LCL_StripTab(LPCTSTR key)
{
	static TCHAR k[128];

	lstrcpy(k,  key);
	TCHAR*  p = k;

	while( *p )
	{
		if( *p < _T(' ') )
		{
			*p = _T('\0');
			break;
		}

		p++;
	}

	return k;
}

// ----------------------------------------------------------------------------
static LPCTSTR LCL_Tabify(LPCTSTR val)
{
	static TCHAR v[128];

	lstrcpy(v,  val);
	TCHAR*  p = v;

	while( *p )
	{
		if( *p == _T('\\') && *(p+1) == _T('t') )
		{
			*p =  _T('\t');
			p++;

			while( *(p+1) )
			{
				*p = *(p+1);
				p++;
			}

			*p = _T('\0');

			break;
		}

		p++;
	}

	return v;
}

// ----------------------------------------------------------------------------
LPCTSTR LCL_String(LPCTSTR sect, LPCTSTR key, bool checkTabs)
{
	LPCTSTR r     = key;
	LPCTSTR k     = checkTabs ? LCL_StripTab(key) : key;
	TCHAR*  gSect = gSects;

	for( int s = 0; s < gNSects; s++ )
	{
		if( lstrcmpi(gSect, sect) == 0 )
		{
			TCHAR* p = gKeys[s];
			int    l = lstrlen(k);

			while( *p )
			{
				if( _tcsnicmp(p, k, l) == 0 )
				{
					if( *(p+l) == _T('=') && *(p+l+1) )
					{
						r = p + l + 1;
						r = checkTabs ? LCL_Tabify(r) : r;
					}

					break;
				}

				p += lstrlen(p) + 1;
			}

			break;
		}

		gSect += lstrlen(gSect) + 1;
	}

	return r;
}
