// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
// more details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
#include "pickFonts.h"
#include "dialogWnd.h"
#include "resource.h"
#include <commctrl.h>
#include "toolbar.h"
#include <stdio.h>

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
static BOOL CALLBACK FontsProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
static int  CALLBACK FontEnumProc(const LOGFONT* pLF, const TEXTMETRIC* pTM, DWORD fontType, LPARAM lParam);

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
void DoPickFonts(HWND hParent, UINT id)
{
	if( CanGame() )
	{
//		BOOL popup = (BOOL)(id == ID_TYPE_ADJUST);
//		CreateDialogParam(GetModuleHandle(NULL), MAKEINTRESOURCE(IDD_PICKFONTS), hParent, FontsProc, (LPARAM)popup);
		CreateDialogParam(GetModuleHandle(NULL), MAKEINTRESOURCE(IDD_PICKFONTS), hParent, FontsProc, NULL);
	}
}

// ----------------------------------------------------------------------------
static BOOL CALLBACK FontsProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch( uMsg )
	{
	case WM_INITDIALOG:
		{
			gameDlg = hDlg;

			LOGFONT lf;
			memset(&lf, 0, sizeof(lf));
			HDC     hDC = GetDC(NULL);
			EnumFontFamiliesEx(hDC, &lf, FontEnumProc, (LPARAM)hDlg, 0);
			ReleaseDC(NULL, hDC);

			Toolbar_BtnCheck(gameTBar, ID_PICK_FONTS, true);
			RECT tr = Toolbar_BtnRect(gameTBar, ID_PICK_FONTS, true);

			return InitDlgProc(_T("PickFontsDialog"), hDlg, tr.right, tr.bottom+1, true);
		}
		break;

	case WM_DESTROY:
		{
			Toolbar_BtnCheck(gameTBar, ID_PICK_FONTS, false);
			SetFocus(GetParent(hDlg));
			gameDlg = NULL;
		}
		break;

	case WM_ACTIVATE:
		{
			if( LOWORD(wParam) == WA_INACTIVE )
			{
				if( GetParent((HWND)lParam) != hDlg )
					DestroyWindow(hDlg);
			}
		}
		break;

	case WM_COMMAND:
		{
			switch( wParam )
			{
			case IDCANCEL:
			case IDOK:
				{
					DestroyWindow(hDlg);
				}
				break;
			}
		}
		break;

	case WM_MEASUREITEM:
		{
			MEASUREITEMSTRUCT* pMIS = (MEASUREITEMSTRUCT*)lParam;
			pMIS->itemHeight = 28;
			return TRUE;
		}
		break;

	case WM_DRAWITEM:
		{
			DRAWITEMSTRUCT* pDIS = (DRAWITEMSTRUCT*)lParam;

			if( pDIS->itemID != -1 )
			{
				LOGFONT lf;
				HDC     hDC    =  GetDC(NULL);
				int     dpi    =  GetDeviceCaps(hDC, LOGPIXELSY);
				int		ok1    =  ReleaseDC(NULL, hDC);
				HFONT   hFont  = (HFONT)GetStockObject(DEFAULT_GUI_FONT);
				int     bytes  =  GetObject(hFont, sizeof(LOGFONT), &lf);

				SendMessage(pDIS->hwndItem, LB_GETTEXT, pDIS->itemID, (LPARAM)lf.lfFaceName);

				lf.lfHeight    = -MulDiv(14, dpi, 72);
				hFont          =  CreateFontIndirect(&lf);
				HGDIOBJ hDCFnt =  SelectObject(pDIS->hDC, hFont);

				SetBkMode(pDIS->hDC, TRANSPARENT);
				DrawText(pDIS->hDC, _T("123456789"), -1, &pDIS->rcItem, DT_CENTER | DT_SINGLELINE | DT_VCENTER);
				SelectObject(pDIS->hDC, hDCFnt);
				DeleteObject(hFont);

				if( pDIS->itemState == ODS_SELECTED )
					DrawFocusRect(pDIS->hDC, &pDIS->rcItem);
			}

			return TRUE;
		}
		break;
	}

	return FALSE;
}

// ----------------------------------------------------------------------------
int CALLBACK FontEnumProc(const LOGFONT* pLF, const TEXTMETRIC* pTM, DWORD fontType, LPARAM lParam)
{
	SendMessage(GetDlgItem((HWND)lParam, IDC_LIST), LB_INSERTSTRING, -1, (LPARAM)pLF->lfFaceName);
	return 1;
}
