// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
// more details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
#include "pickNumber.h"
#include "dialogWnd.h"
#include "resource.h"
#include "main.h"
#include "wnd.h"

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
static BOOL CALLBACK PickNumberProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
void DoPickNumber(HWND hParent)
{
	if( CanGame() )
		CreateDialog(GetModuleHandle(NULL), MAKEINTRESOURCE(IDD_SELCTNO), hParent, PickNumberProc);
}

// ----------------------------------------------------------------------------
static BOOL CALLBACK PickNumberProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch( uMsg )
	{
	case WM_INITDIALOG:
		{
			gameDlg  = hDlg;

			RECT wr  = Wnd_OuterRect(hDlg);
			RECT tbr = Wnd_OuterRect(GetDlgItem(GetParent(hDlg), REBAR));

			POINT pt = { gameCol*(colW + colM) + colM + colW/2,
						 gameRow*(rowH + rowM) + rowM + rowH/2+tbr.bottom-tbr.top };

			wr.right    -= wr.left;
			wr.bottom   -= wr.top;

			if( gameCol == SDSIZESQ/2 ) pt.x -= wr.right /2;
			if( gameRow == SDSIZESQ/2 ) pt.y -= wr.bottom/2;
			if( gameCol >  SDSIZESQ/2 ) pt.x -= wr.right;
			if( gameRow >  SDSIZESQ/2 ) pt.y -= wr.bottom;

			ClientToScreen(GetParent(hDlg), &pt);

			return InitDlgProc(_T("SelectNumberDialog"), hDlg, pt.x, pt.y);
		}
		break;

	case WM_DESTROY:
		{
			SetFocus(GetParent(hDlg));
			gameDlg = NULL;
		}
		break;

	case WM_ACTIVATE:
		{
			if( LOWORD(wParam) == WA_INACTIVE )
			{
				if( GetParent((HWND)lParam) != hDlg )
					DestroyWindow(hDlg);
			}
		}
		break;

	case WM_COMMAND:
		{
			bool done = true;
			int  nbr  = -1;

			switch( wParam )
			{
			case IDCANCEL:
			case IDOK:                          break;
			case IDC_BUTTON1:     nbr  = 1;     break;
			case IDC_BUTTON2:     nbr  = 2;     break;
			case IDC_BUTTON3:     nbr  = 3;     break;
			case IDC_BUTTON4:     nbr  = 4;     break;
			case IDC_BUTTON5:     nbr  = 5;     break;
			case IDC_BUTTON6:     nbr  = 6;     break;
			case IDC_BUTTON7:     nbr  = 7;     break;
			case IDC_BUTTON8:     nbr  = 8;     break;
			case IDC_BUTTON9:     nbr  = 9;     break;
			case IDC_EMPTY:       nbr  = 0;     break;
			case IDC_SHOPOSSIBLE: done = false; CheckDlgButton(hDlg, IDC_DELPOSSIBLE, BST_UNCHECKED); break;
			case IDC_DELPOSSIBLE: done = false; CheckDlgButton(hDlg, IDC_SHOPOSSIBLE, BST_UNCHECKED); break;
			default:              done = false; break;
			}

			if( nbr != -1 )
			{
				bool shoposs = IsDlgButtonChecked(hDlg, IDC_SHOPOSSIBLE) == BST_CHECKED;
				bool remposs = IsDlgButtonChecked(hDlg, IDC_DELPOSSIBLE) == BST_CHECKED;

				if( shoposs || remposs )
				{
					if( nbr )
					gamePoss[gameRow][gameCol][nbr-1] = shoposs ? EMPHASIZE : 0;
				}
				else
				{
					gameCell[gameRow][gameCol].usr    = nbr;
					gameCell[gameRow][gameCol].sho    = nbr > 0 ? true : false;
					gameWon                           = ChkGame(false);
				}

				DrwGame(false);
			}

			if( done )
				DestroyWindow(hDlg);
		}
		break;
	}

	return FALSE;
}
