// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
// more details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
#include "settings.h"
#include "resource.h"
#include <commctrl.h>
#include "toolbar.h"
#include <stdio.h>

static char  profile[MAX_PATH];

static int   initType;
static bool  initChk;
static bool  initSho;
static bool  initTim;
static int   initSeed;
static int   initLevel;
static int   initDiff[gameExtreme+1];
static char  initUser[SDSIZESQ*SDSIZESQ+1];
static char  initGrid[SDSIZESQ*SDSIZESQ+1];
static char  initGive[SDSIZESQ*SDSIZESQ+1];
static char  initPoss[SDSIZESQ][SDSIZESQ*SDSIZESQ+1];

static void  GetFont(int number, LOGFONT& lf, COLORREF& color);

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
bool GetGame(bool draw)
{
	static bool firstCall = true;

	if( firstCall )
	{
		firstCall = false;
		GetCurrentDirectoryA(SIZEOF(profile), profile);
		strcat(profile, "\\USBSudoku.ini");
	}

	ClrGame();

	initType  = GetPrivateProfileIntA("Game", "Difficulty", -1, profile);
	bool okay = initType != -1;

	if( okay )
	{
		GetFont(0, gameLogF1, gameClrF1);
		GetFont(1, gameLogF2, gameClrF2);

		gameType = (GameType)initType;
		gameChk  = initChk = GetPrivateProfileIntA("Game", "Check",     gameChk ? 1 : 0, profile) == 1;
		gameSho  = initSho = GetPrivateProfileIntA("Game", "Possibles", gameSho ? 1 : 0, profile) == 1;
		gameTim  = initTim = GetPrivateProfileIntA("Game", "Timer",     gameTim ? 1 : 0, profile) == 1;

//		Toolbar_BtnCheck(gameTBar, ID_SHOW_ERRORS,    gameChk);
//		Toolbar_BtnCheck(gameTBar, ID_SHOW_POSSIBLES, gameSho);

		LPCSTR labels[] = { "Seed",    "Level",    "Minimum",    "Beginner",   "Easy",       "Medium",     "Hard",       "Extreme"    };
		int*   values[] = { &gameSeed, &gameLevel, &gameDiff[0], &gameDiff[1], &gameDiff[2], &gameDiff[3], &gameDiff[4], &gameDiff[5] };
		int*   inits [] = { &initSeed, &initLevel, &initDiff[0], &initDiff[1], &initDiff[2], &initDiff[3], &initDiff[4], &initDiff[5] };

		for( int d = 0; d < SIZEOF(labels); d++ )
			*values[d] = *inits[d] = GetPrivateProfileIntA("Game", labels[d], *values[d], profile);

		for( int row = 0; row < SDSIZESQ; row++ )
		{
			char skey[] = "Pos0";
			skey[3]    += row + 1;
			GetPrivateProfileStringA("Game", skey, "", &initPoss[row][0], SIZEOF(initPoss[row]), profile);

			for( int col = 0; col < SDSIZESQ; col++ )
			{
				for( int pos = 0; pos < SDSIZESQ; pos++ )
				{
					BYTE gp = initPoss[row][col*SDSIZESQ+pos] - _T('0');
					gamePoss[row][col][pos] = gp;
				}
			}
		}

		LPCSTR types[] = { "User",   "Grid",   "Give"   };
		char*  store[] = { initUser, initGrid, initGive };

		for( int t = 0; t < SIZEOF(types); t++ )
		{
			GetPrivateProfileStringA("Game", types[t], "", store[t], SIZEOF(initUser), profile);
			okay = strlen(store[t]) == SDSIZESQ*SDSIZESQ;
			if( !okay ) break;
		}

		if( okay )
			SetGame(gameSeed, initUser, initGrid, initGive);

		if( gameChk )
		{
			gameChk = !gameChk;
			VerGame(draw, true);
		}
		else
		{
			StaGame(true);
			if( draw )
				DrwGame(false);
		}
	}

	return okay;
}

// ----------------------------------------------------------------------------
static void GetFont(int number, LOGFONT& lf, COLORREF& color)
{
	WCHAR nmap;
	char  font[1024], name[1024];
	char  sect[] = "Font0";
	char  key [] = "Map0";

	sect[4]     +=  number + 1;

	HFONT hFont  = (HFONT)GetStockObject(DEFAULT_GUI_FONT);
	int   bytes  =  GetObject(hFont, sizeof(LOGFONT), &lf);
	int   slen   =  GetPrivateProfileStringA(sect, "Font", "", font, SIZEOF(font), profile);

	lf.lfWeight  =  number == 0 ? FW_BOLD : FW_THIN;

	if( slen )
	{
		sscanf(font, "%d %d %d %d %d %d %d %d %d %d %d %d %d %s",
			   &lf.lfHeight,         &lf.lfWidth,        &lf.lfEscapement,    &lf.lfOrientation,
			   &lf.lfWeight,         &lf.lfItalic,       &lf.lfUnderline,     &lf.lfStrikeOut,
			   &lf.lfCharSet,        &lf.lfOutPrecision, &lf.lfClipPrecision, &lf.lfQuality,
			   &lf.lfPitchAndFamily,  name);

		#ifdef _UNICODE
		int wlen = MultiByteToWideChar(CP_ACP, 0, name, strlen(name), lf.lfFaceName, SIZEOF(lf.lfFaceName));
		lf.lfFaceName[wlen] = L'\0';
		#else
		strcpy(lf.lfFaceName, name);
		#endif
	}

	for( int m = 0; m < SIZEOF(gameMapF1); m++ )
	{
		key[3]++;
		nmap = GetPrivateProfileIntA(sect, key, '1'+m, profile);

		if( number == 0 ) gameMapF1[m] = nmap;
		else              gameMapF2[m] = nmap;
	}

	int r = GetPrivateProfileIntA(sect, "Red",   GetRValue(color), profile);
	int g = GetPrivateProfileIntA(sect, "Green", GetGValue(color), profile);
	int b = GetPrivateProfileIntA(sect, "Blue",  GetBValue(color), profile);
	color = RGB(r, g, b);
}

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
void SavGame()
{
	char   nbr[16];
	int    inits [] = { initType,     initChk ? 1 : 0, initSho ? 1 : 0, initTim ? 1 : 0, initSeed, initLevel, initDiff[0], initDiff[1], initDiff[2], initDiff[3], initDiff[4], initDiff[5] };
	int    values[] = { gameType,     gameChk ? 1 : 0, gameSho ? 1 : 0, gameTim ? 1 : 0, gameSeed, gameLevel, gameDiff[0], gameDiff[1], gameDiff[2], gameDiff[3], gameDiff[4], gameDiff[5] };
	LPCSTR labels[] = { "Difficulty", "Check",         "Possibles",     "Timer",         "Seed",   "Level",   "Minimum",   "Beginner",  "Easy",      "Medium",    "Hard",      "Extreme"   };

	for( int v = 0; v < SIZEOF(values); v++ )
	{
		if( values[v] != inits[v] )
			WritePrivateProfileStringA("Game", labels[v], itoa(values[v], nbr, 10), profile);
	}

	char   grid[SDSIZESQ*SDSIZESQ+1];
	LPCSTR types[] = { "User",   "Grid",   "Give"   };
	char*  store[] = { initUser, initGrid, initGive };

	for( int t = 0; t < SIZEOF(types); t++ )
	{
		SavGame(grid, t);
		grid[SDSIZESQ*SDSIZESQ] = '\0';
		if( strcmp(grid, store[t]) )
			WritePrivateProfileStringA("Game", types[t], grid, profile);
	}

	for( int row = 0; row < SDSIZESQ; row++ )
	{
		for( int col = 0; col < SDSIZESQ; col++ )
		{
			for( int pos = 0; pos < SDSIZESQ; pos++ )
			{
				grid[col*SDSIZESQ+pos] = gamePoss[row][col][pos] + '0';
			}
		}

		if( strcmp(grid, &initPoss[row][0]) )
		{
			char skey[] = "Pos0";
			skey[3]    += row + 1;
			grid[SDSIZESQ*SDSIZESQ] = '\0';
			WritePrivateProfileStringA("Game", skey, grid, profile);
		}
	}
}
