// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
// more details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
#include "shadowWnd.h"
#include "wnd.h"

static LRESULT CALLBACK WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
static LRESULT CALLBACK SubProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
static void PosShadow(HWND hWnd, HWND hShadow, bool move, bool size, bool zorder);

static bool first = true;

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
void CreateShadow(HWND hWnd)
{
	RECT wr = Wnd_OuterRect(hWnd);

	static LPCTSTR SHADOWCLASS = _T("ShadowWndClass");

	if( first )
	{
		first         =   false;
		WNDCLASSEX wc = { sizeof(wc), CS_DROPSHADOW, WndProc, 0, 0, GetModuleHandle(NULL), NULL, NULL, NULL, NULL, SHADOWCLASS, NULL };
		RegisterClassEx(&wc);
	}

	HWND hShadow = CreateWindowEx(WS_EX_NOACTIVATE | WS_EX_LAYERED | WS_EX_TOOLWINDOW | WS_EX_TRANSPARENT,
								  SHADOWCLASS, NULL, WS_POPUP, wr.left, wr.top,
								  wr.right-wr.left, wr.bottom-wr.top, NULL,
								  NULL, GetModuleHandle(NULL), hWnd);
	if( hShadow )
	{
		PosShadow(hWnd, hShadow, true, true, true);
		ShowWindow(hShadow, SW_SHOWNA);
	}
}

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
static LRESULT CALLBACK WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch( uMsg )
	{
	case WM_CREATE:
		{
			CREATESTRUCT* pCS     = (CREATESTRUCT*)lParam;
			HWND          hWatch  = (HWND)pCS->lpCreateParams;
			WNDPROC       wndProc = (WNDPROC)GetWindowLong(hWatch, GWL_WNDPROC);
			BOOL          okay1   = SetProp(hWatch, _T("hShadow"), hWnd);
			BOOL          okay2   = SetProp(hWatch, _T("wndProc"), wndProc);
			SetWindowLong(hWatch, GWL_WNDPROC, (LONG)SubProc);
		}
		break;
	}

	return DefWindowProc(hWnd, uMsg, wParam, lParam);
}

// ----------------------------------------------------------------------------
static LRESULT CALLBACK SubProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	HWND    hShadow = (HWND)   GetProp(hWnd, _T("hShadow"));
	WNDPROC wndProc = (WNDPROC)GetProp(hWnd, _T("wndProc"));

	switch( uMsg )
	{
	case WM_DESTROY:
		{
			RemoveProp(hWnd, _T("hShadow"));
			RemoveProp(hWnd, _T("wndProc"));

			if( hShadow ) DestroyWindow(hShadow);
		}
		break;

	case WM_MOVE:
		{
			if( hShadow )
				PosShadow(hWnd, hShadow, true, false, false);
		}
		break;

	case WM_SIZE:
		{
			if( hShadow )
				PosShadow(hWnd, hShadow, false, true, false);
		}
		break;

	case WM_WINDOWPOSCHANGED:
		{
			if( hShadow )
			{
				LPWINDOWPOS pWP    = (LPWINDOWPOS)lParam;
				bool        move   = (pWP->flags & SWP_NOMOVE)   != SWP_NOMOVE;
				bool        size   = (pWP->flags & SWP_NOSIZE)   != SWP_NOSIZE;
				bool        zorder = (pWP->flags & SWP_NOZORDER) != SWP_NOZORDER;
				PosShadow(hWnd, hShadow, move, size, zorder);
			}
		}
		break;
	}

	if( wndProc )
		return CallWindowProc(wndProc, hWnd, uMsg, wParam, lParam);

	return DefWindowProc(hWnd, uMsg, wParam, lParam);
}

// ----------------------------------------------------------------------------
static void PosShadow(HWND hWnd, HWND hShadow, bool move, bool size, bool zorder)
{
	RECT wr     = Wnd_OuterRect(hWnd);
	HWND hAfter = GetWindow(hWnd, GW_HWNDPREV);
	UINT flags  = SWP_NOACTIVATE;
	flags      |= move   ? 0 : SWP_NOMOVE;
	flags      |= size   ? 0 : SWP_NOSIZE;
	flags      |= zorder ? 0 : SWP_NOZORDER;

	if( size )
	{
		HRGN hRgn =  CreateRectRgn(0, 0, 0, 0);

		if(  hRgn && GetWindowRgn(hWnd,    hRgn) == COMPLEXREGION )
					 SetWindowRgn(hShadow, hRgn, TRUE);
	}

	SetWindowPos(hShadow, hAfter, wr.left, wr.top, wr.right-wr.left, wr.bottom-wr.top, flags);
}
