// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
// more details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
#include "system.h"

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
void Sys_AppendIcon(HIMAGELIST hIL, LPCTSTR file, UINT resID, int size, COLORREF bkgnd)
{
	HICON hIcon = (HICON)LoadImage(NULL, file, IMAGE_ICON, size, size, LR_LOADFROMFILE | LR_CREATEDIBSECTION);

	if(  !hIcon )
		  hIcon = (HICON)LoadImage(GetModuleHandle(NULL), MAKEINTRESOURCE(resID), IMAGE_ICON, size, size, LR_DEFAULTCOLOR | LR_CREATEDIBSECTION);

//	if( hIcon && Sys_OSVersion() == osWin2K )
	if( hIcon && Sys_OSVersion() <= osWin2K )
	{
		ICONINFO ii;
		BITMAP   bmColor32, bmColor24, bmMaskr24;

		GetIconInfo(hIcon, &ii);
		GetObject(ii.hbmColor, sizeof(bmColor32), &bmColor32);

		if( bmColor32.bmBitsPixel == 32 )
		{
			DestroyIcon(hIcon);
			DeleteObject(ii.hbmMask);

			hIcon      = NULL;
			ii.hbmMask = NULL;

			UINT       a32;
			int        o24, o32;
			VOID*      pBits    = NULL;
			UINT       r24bkgnd = (UINT)GetRValue(bkgnd);
			UINT       g24bkgnd = (UINT)GetGValue(bkgnd);
			UINT       b24bkgnd = (UINT)GetBValue(bkgnd);
			BITMAPINFO bmi      = { sizeof(BITMAPINFOHEADER), bmColor32.bmWidth, bmColor32.bmHeight, 1, 32, BI_RGB, 0, 0, 0, 0, 0 };
//			BITMAPINFO bmi      = { sizeof(BITMAPINFOHEADER), bmColor32.bmWidth, bmColor32.bmHeight, 1, 24, BI_RGB, 0, 0, 0, 0, 0 };
			HBITMAP    hColor32 = (HBITMAP)CopyImage(ii.hbmColor, IMAGE_BITMAP, 0, 0, LR_COPYDELETEORG | LR_COPYRETURNORG | LR_CREATEDIBSECTION);

			ii.hbmColor = NULL;

			if( hColor32 )
			{
				ii.hbmColor = CreateDIBSection(NULL, &bmi, DIB_RGB_COLORS, &pBits, NULL, 0);
				ii.hbmMask  = CreateDIBSection(NULL, &bmi, DIB_RGB_COLORS, &pBits, NULL, 0);

				if( ii.hbmColor && ii.hbmMask )
				{
					GetObject(hColor32,    sizeof(bmColor32), &bmColor32);
					GetObject(ii.hbmColor, sizeof(bmColor24), &bmColor24);
					GetObject(ii.hbmMask,  sizeof(bmMaskr24), &bmMaskr24);

					if( bmColor32.bmBits && bmColor24.bmBits && bmMaskr24.bmBits )
					{
						for( int y = 0; y < bmColor32.bmHeight; y++ )
						{
							BYTE* pColor32 = (BYTE*)bmColor32.bmBits + y*bmColor32.bmWidthBytes;
							BYTE* pColor24 = (BYTE*)bmColor24.bmBits + y*bmColor24.bmWidthBytes;
							BYTE* pMaskr24 = (BYTE*)bmMaskr24.bmBits + y*bmMaskr24.bmWidthBytes;

							for( int x = 0; x < bmColor32.bmWidth; x++ )
							{
								o24 =  x*4;
//								o24 =  x*3;
								o32 =  x*4;
								a32 = (UINT)pColor32[o32+3];
								pMaskr24[o24+0] = pMaskr24[o24+1] = pMaskr24[o24+2] = pColor32[o32+3] < 1 ? 255 : 0;
								pColor24[o24+0] = (BYTE)((UINT)pColor32[o32+0]*a32/255) + (BYTE)(b24bkgnd*(255-a32)/255);
								pColor24[o24+1] = (BYTE)((UINT)pColor32[o32+1]*a32/255) + (BYTE)(g24bkgnd*(255-a32)/255);
								pColor24[o24+2] = (BYTE)((UINT)pColor32[o32+2]*a32/255) + (BYTE)(r24bkgnd*(255-a32)/255);
							}
						}

						hIcon = CreateIconIndirect(&ii);
					}
				}

				if( hColor32 ) DeleteObject(hColor32);
			}
		}

		if( ii.hbmColor ) DeleteObject(ii.hbmColor);
		if( ii.hbmMask  ) DeleteObject(ii.hbmMask);
	}

	int iL = ImageList_ReplaceIcon(hIL, -1, hIcon);
	DestroyIcon(hIcon);
}

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
SIZE Sys_BitmapSize(HBITMAP hBitmap)
{
	BITMAP    bm;
	GetObject(hBitmap, sizeof(bm), &bm);
	SIZE      sz = { bm.bmWidth, bm.bmHeight };
	return    sz;
}

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
#include <shlwapi.h>
static float ccv = 0.0f;

// ----------------------------------------------------------------------------
float Sys_ComCtlVersion()
{
	if( ccv != 0.0f ) return ccv;

	HINSTANCE hComCtl = GetModuleHandle(_T("comctl32.dll"));
	ccv               = 4.0f;

	if( hComCtl )
	{
		DLLGETVERSIONPROC pDllGetVersion = (DLLGETVERSIONPROC)GetProcAddress(hComCtl, "DllGetVersion");

		if( pDllGetVersion )
		{
			DLLVERSIONINFO dvi = { sizeof(dvi), 0, 0, 0, 0 };
			HRESULT        hr  = (*pDllGetVersion)(&dvi);

			if( SUCCEEDED(hr) )
			{
				char    nbr[32];
				sprintf(nbr, "%d.%d", dvi.dwMajorVersion, dvi.dwMinorVersion);
				ccv =  (float)atof(nbr);
			}
		}
	}

	return ccv;
}

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
LRESULT Sys_HandleMouseWheel(HWND hWnd, WPARAM wParam, LPARAM lParam, bool checkMP, bool single)
{
	if( checkMP )
	{
		POINT mp     = { GET_X_LPARAM(lParam), GET_Y_LPARAM(lParam) };
		HWND  hWndMP = WindowFromPoint(mp);

		if( GetWindowThreadProcessId(hWndMP, NULL) == GetCurrentThreadId() )
			hWnd = hWndMP;
	}

	int  z = GET_WHEEL_DELTA_WPARAM(wParam);
	int  n = z / WHEEL_DELTA;
	bool d = n < 0;
	WORD c = d ? SB_LINEDOWN : SB_LINEUP;

	n      = abs(n);

	while( n-- )
	{
		PostMessage(hWnd, WM_VSCROLL, c, 0);
		if( single ) break;
	}

	return 0; // so that the msg does not get propagated to the parent
}

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
bool Sys_KeyIsDown(int key)
{
	return (GetAsyncKeyState(key) & 0x8000) == 0x8000;
}

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
static tOSVersion osv = osNotInitd;

// ----------------------------------------------------------------------------
tOSVersion Sys_OSVersion()
{
	if( osv != osNotInitd ) return osv;

	OSVERSIONINFOEX osvi;
	BOOL OSVersionInfoEx;

	ZeroMemory(&osvi, sizeof(OSVERSIONINFOEX));
	osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFOEX);

	if( !(OSVersionInfoEx = GetVersionEx((OSVERSIONINFO*)&osvi)) )
	{
		osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
		if( !GetVersionEx((OSVERSIONINFO*)&osvi) ) return osv = osUnknown;
	}

	switch( osvi.dwPlatformId )
	{
	case VER_PLATFORM_WIN32_NT:

		if( osvi.dwMajorVersion <= 4 )                              return osv = osWinNT;
		if( osvi.dwMajorVersion == 5 && osvi.dwMinorVersion ==  0 ) return osv = osWin2K;
		if( osvi.dwMajorVersion == 5 && osvi.dwMinorVersion ==  1 ) return osv = osWinXP;
		if( osvi.dwMajorVersion == 5 && osvi.dwMinorVersion ==  2 ) return osv = osWinS2K3;
		if( osvi.dwMajorVersion == 5 && osvi.dwMinorVersion >   2 ) return osv = osWinFuture;
		break;

	case VER_PLATFORM_WIN32_WINDOWS:

		if( osvi.dwMajorVersion == 4 && osvi.dwMinorVersion == 0  ) return osv = osWin95;
		if( osvi.dwMajorVersion == 4 && osvi.dwMinorVersion == 10 ) return osv = osWin98;
		if( osvi.dwMajorVersion == 4 && osvi.dwMinorVersion == 90 ) return osv = osWinME;
		break;
   }

   return osv = osUnknown; 
}

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
void Sys_ResetMemUsage()
{
	HMODULE hKernel32 = GetModuleHandle(_T("kernel32.dll"));

	if( hKernel32 )
	{
		typedef DWORD  (__stdcall *pGetProcessHeaps)         (DWORD,  PHANDLE);
		typedef HANDLE (__stdcall *pGetProcessHeap)          (VOID);
		typedef UINT   (__stdcall *pHeapCompact)             (HANDLE, DWORD);
		typedef BOOL   (__stdcall *pSetProcessWorkingSetSize)(HANDLE, DWORD, DWORD);

		#define MAXHEAPS 16

		pGetProcessHeaps pGPHs = (pGetProcessHeaps)GetProcAddress(hKernel32, "GetProcessHeaps");
		pGetProcessHeap  pGPH  = (pGetProcessHeap) GetProcAddress(hKernel32, "GetProcessHeap");
		pHeapCompact     pHC   = (pHeapCompact)    GetProcAddress(hKernel32, "HeapCompact");

		if( pGPHs && pGPH && pHC )
		{
			int numHeaps1 = pGPHs(0, NULL);

			if( numHeaps1 > MAXHEAPS )
				pHC(pGPH(), 0);
			else
			{
				HANDLE heaps[MAXHEAPS];
				int numHeaps2 = pGPHs(numHeaps1, heaps);

				for( int h = 0; h < numHeaps1; h++ )
					pHC(heaps[h], 0);
			}
		}

		pSetProcessWorkingSetSize pSPWSS = (pSetProcessWorkingSetSize)GetProcAddress(hKernel32, "SetProcessWorkingSetSize");

		if( pSPWSS )
			pSPWSS(GetCurrentProcess(), -1, -1);
	}
}

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
void Sys_SendKeys(HWND hWnd, LPCTSTR string)
{
	BYTE  vk;
	SHORT ks;
	bool  sk = false;
	bool  ck = false;
	bool  ak = false;

//	Activate(hWnd);

	while(  *string )
	{
		vk = -1;
		sk = false;
		ck = false;
		ak = false;

		if( *string == _T('<') )
		{
			string  =  _tcschr(string, _T('>'));
			if( !string ) break;

			switch( *(string-1) )
			{
			case _T('a'): vk = VK_MENU;    break; // alt
			case _T('c'): vk = VK_CONTROL; break;
			case _T('s'): vk = VK_SHIFT;   break;
			case _T('e'): vk = VK_RETURN;  break; // enter
			case _T('b'): vk = VK_BACK;    break; // backspace
			case _T('u'): vk = VK_UP;      break; // up    arrow
			case _T('d'): vk = VK_DOWN;    break; // down  arrow
			case _T('l'): vk = VK_LEFT;    break; // left  arrow
			case _T('r'): vk = VK_RIGHT;   break; // right arrow
			}
		}
		else
		{
			ks =  VkKeyScan(*string);
			vk =  LOBYTE(ks);
			sk = (HIBYTE(ks) & 1) == 1;
			ck = (HIBYTE(ks) & 2) == 2;
			ak = (HIBYTE(ks) & 4) == 4;
		}

		if( sk )
		keybd_event(VK_SHIFT, 0,  0,               0); // shift key down

		if( vk != -1 )
		{
			keybd_event(vk,   0,  0,               0); // string key down
			keybd_event(vk,   0,  KEYEVENTF_KEYUP, 0); // string key up
		}

		if( sk )
		keybd_event(VK_SHIFT, 0,  KEYEVENTF_KEYUP, 0); // shift key up

		string++;
	}
}
