// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
// more details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
#include "toolbar.h"
#include "resource.h"
#include <commctrl.h>
#include "system.h"
#include "rebar.h"
#include "wnd.h"

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
HWND Toolbar_Create(HWND hParent, UINT id)
{
	HWND hCtl = CreateWindowEx(0, TOOLBARCLASSNAME, NULL, CCS_NORESIZE | TBSTYLE_FLAT | TBSTYLE_LIST | TBSTYLE_TRANSPARENT | WS_CHILD | WS_CLIPCHILDREN | WS_CLIPSIBLINGS | WS_VISIBLE, 0, 0, 0, 0, hParent, (HMENU)id, GetModuleHandle(NULL), NULL);

	UINT       style  = BTNS_BUTTON   | BTNS_AUTOSIZE;
	UINT       stylec = BTNS_CHECK    | style;
	UINT       styled = BTNS_DROPDOWN | style;
	UINT       styleD = BTNS_DROPDOWN | stylec;
	bool       ie581  = Sys_ComCtlVersion() >= 5.81f;
	HIMAGELIST hIL    = ImageList_Create(32, 32, ILC_COLOR32, 0, 1);
	LPCTSTR    tbs [] = { _T("E&xit"),    _T("&Previous"), _T("&Next"),    _T("&Restart"),    _T("&Solve"),    _T("&Difficulty"),    _T("&Check"),    _T("&Possibles"),    _T("&Fonts")    };
	LPCTSTR    tbi [] = { _T("exit.ico"), _T("prev.ico"),  _T("next.ico"), _T("restart.ico"), _T("solve.ico"), _T("difficulty.ico"), _T("check.ico"), _T("possibles.ico"), _T("fonts.ico") };
	UINT       tbid[] = { IDI_EXIT,       IDI_PREV,        IDI_NEXT,       IDI_RESTART,       IDI_SOLVE,       IDI_DIFFICULTY,       IDI_CHECK,       IDI_POSSIBLES,       IDI_FONTS       };
	TBBUTTON   tbb [] =
	{
		0, ID_GAME_EXIT,      TBSTATE_ENABLED, style,    0, 0, 0, ie581 ? (int)tbs[0] : 0,
		0, 0,                 0,               BTNS_SEP, 0, 0, 0, 0,
		1, ID_GAME_PREV,      TBSTATE_ENABLED, style,    0, 0, 0, ie581 ? (int)tbs[1] : 0,
		2, ID_GAME_NEXT,      TBSTATE_ENABLED, style,    0, 0, 0, ie581 ? (int)tbs[2] : 0,
		0, 0,                 0,               BTNS_SEP, 0, 0, 0, 0,
		3, ID_GAME_RESTART,   TBSTATE_ENABLED, style,    0, 0, 0, ie581 ? (int)tbs[3] : 0,
		4, ID_GAME_SOLVE,     TBSTATE_ENABLED, style,    0, 0, 0, ie581 ? (int)tbs[4] : 0,
		0, 0,                 0,               BTNS_SEP, 0, 0, 0, 0,
		5, ID_DIFFICULTY,     TBSTATE_ENABLED, styleD,   0, 0, 0, ie581 ? (int)tbs[5] : 0,
//		6, ID_SHOW_ERRORS,    TBSTATE_ENABLED, styleD,   0, 0, 0, ie581 ? (int)tbs[6] : 0,
//		7, ID_SHOW_POSSIBLES, TBSTATE_ENABLED, styleD,   0, 0, 0, ie581 ? (int)tbs[7] : 0
		6, ID_SHOW_ERRORS,    TBSTATE_ENABLED, style,    0, 0, 0, ie581 ? (int)tbs[6] : 0,
		7, ID_SHOW_POSSIBLES, TBSTATE_ENABLED, style,    0, 0, 0, ie581 ? (int)tbs[7] : 0,
		0, 0,                 0,               BTNS_SEP, 0, 0, 0, 0,
		8, ID_PICK_FONTS,     TBSTATE_ENABLED, styleD,   0, 0, 0, ie581 ? (int)tbs[8] : 0
	};

	SendMessage(hCtl, TB_BUTTONSTRUCTSIZE, sizeof(TBBUTTON), 0);
	SendMessage(hCtl, TB_SETIMAGELIST,     0,               (LPARAM)(hIL));
	SendMessage(hCtl, TB_SETEXTENDEDSTYLE, 0,               (LPARAM)(TBSTYLE_EX_DRAWDDARROWS | (ie581 ? TBSTYLE_EX_MIXEDBUTTONS : 0)));

//	COLORREF bkgnd = Rebar_BkColor(hParent);
	COLORREF bkgnd = GetSysColor(COLOR_3DFACE);

	for( int i = 0; i < SIZEOF(tbi); i++ )
		Sys_AppendIcon(hIL, tbi[i], tbid[i], 32, bkgnd);

	SendMessage(hCtl, TB_ADDBUTTONS, SIZEOF(tbb), (LPARAM)tbb);
	SendMessage(hCtl, TB_AUTOSIZE, 0, 0);
	SIZE s = Toolbar_MaxSize(hCtl);
	s.cy  += 3; // for spacing

	Rebar_AppendBand(hParent, hCtl, s);

	Toolbar_BtnCheck(hCtl, ID_SHOW_ERRORS,    gameChk);
	Toolbar_BtnCheck(hCtl, ID_SHOW_POSSIBLES, gameSho);

	return hCtl;
}

// ----------------------------------------------------------------------------
void Toolbar_BtnCheck(HWND hCtl, UINT btnID, bool check)
{
	SendMessage(hCtl, TB_CHECKBUTTON, btnID, MAKELPARAM(check ? TRUE : FALSE, 0));
}

// ----------------------------------------------------------------------------
RECT Toolbar_BtnRect(HWND hCtl, UINT btnID, bool screen)
{
	RECT   br;
	SendMessage(hCtl, TB_GETRECT, btnID, (LPARAM)&br);

	if( screen )
	{
		RECT wr    = Wnd_OuterRect(hCtl);
		br.left   += wr.left;
		br.right  += wr.left;
		br.top    += wr.top;
		br.bottom += wr.top;
	}

	return br;
}

// ----------------------------------------------------------------------------
SIZE Toolbar_MaxSize(HWND hCtl)
{
	SIZE size;
//	SendMessage(hCtl, TB_GETMAXSIZE, 0, (LPARAM)&size); // doesn't work correctly with 2K toolbar

	RECT r;
	int  nb = SendMessage(hCtl, TB_BUTTONCOUNT, 0, 0);
	BOOL ok = SendMessage(hCtl, TB_GETITEMRECT, nb-1, (LPARAM)&r);
	size.cx = r.right;
	size.cy = r.bottom - r.top;

	return size;
}
