// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
// more details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
#include "main.h"
#include <stdio.h>
#include <commctrl.h>

#include "wnd.h"
#include "rebar.h"
#include "toolbar.h"
#include "statusbar.h"
#include "showHelp.h"
#include "showAbout.h"
#include "adjustDifficulty.h"
#include "pickNumber.h"
#include "pickFonts.h"
#include "pickGame.h"
#include "settings.h"
#include "resource.h"
#include "system.h"
#include "locale.h"
#include "sudoku.h"

// ----------------------------------------------------------------------------
static LRESULT CALLBACK WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
static DWORD   WINAPI   ExeGameThread(void* lParam);

static void BegGame(bool draw, GameType type, bool force=false);
static void EndGame(bool draw);
static void ExeGame(bool newGame);
static void GetGameBoardBmap(int colW, int rowH, int wndW, int wndH);
static void GetGameBoardFont(int rowH, HFONT& font1, HFONT& font2);
static void GetGameBoardSize(const RECT& cr, int& colM, int& rowM, int& subM, int& colW, int& rowH, int& colH, int& rowW);
static void LodGame();
static void NxtGame(bool draw);
static void PrvGame(bool draw);
static void ShoGame(bool draw);
static void TimGame(TCHAR* ctime, int ltime, bool timeGame, bool first);
static void XitGame();

BYTE     gamePoss[SDSIZESQ][SDSIZESQ][SDSIZESQ]; // row, col, pos(sible) (0, 1, or 2)

HWND     gameWnd      =   NULL;
HWND     gameDlg      =   NULL;
HWND     gameSBar     =   NULL;
HWND     gameRBar     =   NULL;
HWND     gameTBar     =   NULL;
LOGFONT  gameLogF1    = { NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL };
LOGFONT  gameLogF2    = { NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL };
WCHAR    gameMapF1[9] = { NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL };
WCHAR    gameMapF2[9] = { NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL };
COLORREF gameClrF1    =   RGB(0, 0, 0);
COLORREF gameClrF2    =   RGB(0, 0, 0);
HFONT    gameFont1    =   NULL;
HFONT    gameFont2    =   NULL;
HBITMAP  gameEmpty    =   NULL;
HBITMAP  gameFixed    =   NULL;
HBITMAP  gameHlite    =   NULL;
HBITMAP  gameSolve    =   NULL;
HBITMAP  gameCheck    =   NULL;
HBITMAP  gameError    =   NULL;
HBITMAP  gameNumbr[9] = { NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL };
HBITMAP  gameBkgnd    =   NULL;
HBITMAP  drawEmpty    =   NULL;
HBITMAP  drawFixed    =   NULL;
HBITMAP  drawHlite    =   NULL;
HBITMAP  drawSolve    =   NULL;
HBITMAP  drawCheck    =   NULL;
HBITMAP  drawError    =   NULL;
HBITMAP  drawNumbr[9] = { NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL };
HBITMAP  drawBkgnd    =   NULL;
bool     gameExe      =   false;
bool     gameChk      =   false;
bool     gameSho      =   false;
bool     gameTim      =   false;
BYTE     gamePos      =   0;        // 1 = de-emphasize, 2=emphasize
int      colM         =   2;        // margin between puzzle piece columns (except between sub-grids)
int      rowM         =   2;        // margin between puzzle piece rows    (except between sub-grids)
int      subM         =   SDMARGIN; // margin between sub-grids
int      colW         =   32;       // puzzle piece width
int      rowH         =   32;       // puzzle piece height
int      colH         =   0;        // unknown until runtime
int      rowW         =   0;        // unknown until runtime

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
WINAPI _tWinMain(HINSTANCE hInstance, HINSTANCE hPrevInst, LPSTR cmdLine, int show)
{
	int lrgIcoSz  = GetSystemMetrics(SM_CXICON);
	int smlIcoSz  = GetSystemMetrics(SM_CXSMICON);

	WNDCLASSEX wc =
	{
		sizeof(wc),
		CS_DBLCLKS | (Sys_OSVersion() >= osWinXP ? CS_DROPSHADOW : 0),
		WndProc,
		0,
		0,
		hInstance,
		(HICON)LoadImage(hInstance, MAKEINTRESOURCE(IDI_SUDOKU), IMAGE_ICON, lrgIcoSz, lrgIcoSz, LR_DEFAULTCOLOR),
		LoadCursor(NULL, IDC_ARROW),
		NULL,
		MAKEINTRESOURCE(IDR_SUDOKU),
		_T("USBSudokuWndClass"),
		(HICON)LoadImage(hInstance, MAKEINTRESOURCE(IDI_SUDOKU), IMAGE_ICON, smlIcoSz, smlIcoSz, LR_DEFAULTCOLOR)
	};

	RegisterClassEx(&wc);

	INITCOMMONCONTROLSEX  icc = { sizeof(icc), ICC_BAR_CLASSES | ICC_COOL_CLASSES };
	InitCommonControlsEx(&icc);

	LCL_Load(_T(".\\"));

	LodGame();

	if( GetGame(false) == false )
		NewGame(false);

	RECT  r     = { 0, 0, 512, 512+43+20 };
	DWORD style =   WS_CLIPCHILDREN | WS_OVERLAPPEDWINDOW;
	gameWnd     =   CreateWindowEx(0, wc.lpszClassName, _T("USB Sudoku"), style, 0, 0, r.right-r.left, r.bottom-r.top, NULL, NULL, hInstance, NULL);

	ShowWindow(gameWnd, show);

	Sys_ResetMemUsage();

	StaGame(false);

	MSG msg;

	while( GetMessage(&msg, NULL, 0, 0) )
	{
		if( gameDlg && IsDialogMessage(gameDlg, &msg) )
			;
		else
		{
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	}

	SavGame();
	XitGame();

	LCL_Unload();

	return 0;
}

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
static void SetMenuLocale(HWND hWnd)
{
	UINT    cmd;
	TCHAR   name[128], str[128];

	LPCTSTR strn  = NULL;
	HMENU   hMenu = GetMenu(hWnd);
	int     ni    = GetMenuItemCount(hMenu);

	for( int i = 0; i < ni; i++ )
	{
		GetMenuString(hMenu, i, name, SIZEOF(name), MF_BYPOSITION);

		if( cmd  = GetMenuItemID(hMenu, i) )
		{
			strn = LCL_String(name, name, true);
			ModifyMenu(hMenu, i, MF_BYPOSITION | MF_STRING, cmd, strn);

			HMENU hSubMenu = GetSubMenu(hMenu, i);
			int   nsi      = GetMenuItemCount(hSubMenu);

			for( int si = 0; si < nsi; si++ )
			{
				GetMenuString(hSubMenu, si, str, SIZEOF(str), MF_BYPOSITION);

				if( cmd  = GetMenuItemID(hSubMenu, si) )
				{
					strn = LCL_String(name, str, true);
					ModifyMenu(hSubMenu, si, MF_BYPOSITION | MF_STRING, cmd, strn);
				}
			}
		}
	}
}

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
static LRESULT CALLBACK WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	LRESULT ret = -1;

	switch( uMsg )
	{
	case WM_NCCREATE:
		{
			gameWnd = hWnd;
			SetMenuLocale(hWnd);
		}
		break;

	case WM_CREATE:
		{
			gameSBar = Statusbar_Create(gameWnd,  STATUSBAR);
			gameRBar = Rebar_Create    (gameWnd,  REBAR);
			gameTBar = Toolbar_Create  (gameRBar, TOOLBAR);

			RECT cr  = Wnd_InnerRect(gameWnd);
			RECT tbr = Wnd_OuterRect(gameRBar);
			RECT sbr = Wnd_OuterRect(gameSBar);

			int tbh   = tbr.bottom - tbr.top;
			int sbh   = sbr.bottom - sbr.top;
			cr.top    = cr.top     + tbh;
			cr.bottom = cr.bottom  - sbh;

			GetGameBoardSize(cr, colM, rowM, subM, colW, rowH, colH, rowW);
			GetGameBoardBmap(colW, rowH, cr.right-cr.left, cr.bottom-cr.top);

			cr.top    = cr.top    - tbh;
			cr.bottom = cr.bottom + sbh;
			cr.right  = cr.left   + rowW;
			cr.bottom = cr.top    + colH + tbh + sbh;

			AdjustWindowRectEx(&cr, GetWindowLong(hWnd, GWL_STYLE), TRUE, GetWindowLong(hWnd, GWL_EXSTYLE));

			int sw = GetSystemMetrics(SM_CXSCREEN);
			int sh = GetSystemMetrics(SM_CYSCREEN);
			int w  = cr.right  - cr.left;
			int h  = cr.bottom - cr.top;
			int x  = (sw-w)/2;
			int y  = (sh-h)/2;

			GetGameBoardFont(rowH, gameFont1, gameFont2);

			SetWindowPos(hWnd, NULL, x, y, w, h, SWP_NOZORDER);
			SetTimer(hWnd, TIMER, TIMERRES, NULL);
		}
		break;

	case WM_DESTROY:
		{
			KillTimer(hWnd, TIMER);
			KillTimer(hWnd, DBLCLKCHK);
			PostQuitMessage(0);
		}
		break;

	case WM_ACTIVATE:
		{
			if( gameDlg )
			{
				if( LOWORD(wParam) == WA_INACTIVE )
				{
					if( GetParent((HWND)lParam) != gameDlg )
						DestroyWindow(gameDlg);
				}
			}
		}
		break;

	case WM_NOTIFY:
		{
			LPNMHDR pNMH = (LPNMHDR)lParam;

			switch( pNMH->code )
			{
			case NM_CLICK:
				{
					switch( pNMH->idFrom )
					{
					case STATUSBAR:
						{
							LPNMMOUSE pNMM = (LPNMMOUSE)lParam;

							if( pNMM->dwItemSpec == 2 )
							{
								gameTim = !gameTim;
							}
						}
						break;
					}
				}
				break;

			case TBN_DROPDOWN:
				{
					NMTOOLBAR* pTB = (NMTOOLBAR*)lParam;

					if( pTB->iItem == ID_DIFFICULTY )
					{
						TCHAR text[128];
						HMENU hSubMenu = GetSubMenu(GetMenu(hWnd), 1);
						HMENU hPopMenu = CreatePopupMenu();

						MENUITEMINFO mii = { sizeof(mii), MIIM_ID | MIIM_STATE | MIIM_TYPE, 0, 0, 0, 0, NULL, NULL, 0, text, SIZEOF(text) };

						for( int i = 0; i < GetMenuItemCount(hSubMenu); i++ )
						{
							mii.cch = SIZEOF(text);
							GetMenuItemInfo(hSubMenu, i, TRUE, &mii);
							if( (mii.fType & MFT_SEPARATOR) == MFT_SEPARATOR ) break;
							InsertMenuItem (hPopMenu, i, TRUE, &mii);
						}

						RECT tr = Toolbar_BtnRect(pTB->hdr.hwndFrom, pTB->iItem, true);
						TrackPopupMenu(hPopMenu, 0, tr.left, tr.bottom+1, 0, hWnd, NULL);
						DestroyMenu   (hPopMenu);
					}

					ret = TBDDRET_DEFAULT;
				}
				break;
			}
		}
		break;

	case WM_ERASEBKGND:
		{
			DrwGame(false);
			ret = 1;
		}
		break;

	case WM_PAINT:
		{
			PAINTSTRUCT ps;
			BeginPaint(hWnd, &ps);
			DrwGame(false, ps.hdc);
			EndPaint(hWnd, &ps);
		}
		break;

	case WM_SIZE:
		{
			SendMessage(gameRBar, uMsg, wParam, lParam);
			SendMessage(gameSBar, uMsg, wParam, lParam);

			if( wParam == SIZE_MAXIMIZED || wParam == SIZE_RESTORED )
			{
				RECT tbr  =   Wnd_OuterRect(gameRBar);
				RECT sbr  =   Wnd_OuterRect(gameSBar);
				RECT cr   = { 0, 0, LOWORD(lParam), HIWORD(lParam) };
				int  tbh  =   tbr.bottom - tbr.top;
				int  sbh  =   sbr.bottom - sbr.top;
				cr.top    =   cr.top     + tbh;
				cr.bottom =   cr.bottom  - sbh;

				GetGameBoardSize(cr, colM, rowM, subM, colW, rowH, colH, rowW);
				GetGameBoardFont(rowH, gameFont1, gameFont2);
				GetGameBoardBmap(colW, rowH, cr.right-cr.left, cr.bottom-cr.top);
			}

			DrwGame(false);
		}
		break;

	case WM_GETMINMAXINFO:
		{
			RECT cr = Wnd_InnerRect(hWnd);
			RECT wr = Wnd_OuterRect(hWnd);
			SIZE sz = Toolbar_MaxSize(gameTBar);
			RECT bb = Rebar_BandBorders(gameRBar);

			if( IsRectEmpty(&wr) ) break;

			int  w  =   wr.right  - wr.left - (cr.right  - cr.left);
			int  h  =   wr.bottom - wr.top  - (cr.bottom - cr.top);
			RECT r  = { 0, 0, bb.left*4 + sz.cx, sz.cy };

			int  rw, ch, d;
			GetGameBoardSize(r, d, d, d, d, d, ch, rw);

			LPMINMAXINFO pMMI      = (LPMINMAXINFO)lParam;
			pMMI->ptMinTrackSize.x =  w + rw;
			pMMI->ptMinTrackSize.y =  h;
			return 0;
		}
		break;

	case WM_COMMAND:
		{
			switch( LOWORD(wParam) )
			{
			case ID_GAME_LOAD:      GetGame(true);                 break;
			case ID_GAME_SAVE:      SavGame();                     break;
			case ID_GAME_PREV:      PrvGame(true);                 break;
			case ID_GAME_NEXT:      NxtGame(true);                 break;
			case ID_GAME_RANDOM:    RndGame(true);                 break;
			case ID_GAME_RESTART:   BegGame(true, gameType, true); break;
			case ID_GAME_SOLVE:     EndGame(true);                 break;
			case ID_GAME_EXIT:      DestroyWindow(hWnd);           break;

			case ID_TYPE_BEGINNER:  BegGame(true, gameBeginner);   break;
			case ID_TYPE_EASY:      BegGame(true, gameEasy);       break;
			case ID_TYPE_MEDIUM:    BegGame(true, gameMedium);     break;
			case ID_TYPE_HARD:      BegGame(true, gameHard);       break;
			case ID_TYPE_EXTREME:   BegGame(true, gameExtreme);    break;

			case ID_SHOW_ERRORS:    VerGame(true, false);          break;
			case ID_SHOW_POSSIBLES: ShoGame(true);                 break;

			case ID_TYPE_ADJUST:
			case ID_DIFFICULTY:     DoAdjustDiff(hWnd, wParam);    break;
			case ID_PICK_FONTS:     DoPickFonts (hWnd, wParam);    break;
			case ID_GAME_NUMBER:    DoPickGame  (hWnd);            break;
			case ID_ABOUT:          DoShowAbout (hWnd);            break;
			case ID_GAME_HELP:      DoShowHelp  (hWnd);            break;
			}
		}
		break;

	case WM_CHAR:
		{
			if( gameDlg ) break;

			switch( wParam )
			{
			case _T('_'):
			case _T('='):
			case _T('-'):
			case _T('+'):
				{
					if( !gameWon && !gameCell[gameRow][gameCol].fix )
					{
						gamePos = wParam == _T('-') ||
								  wParam == _T('_')  ? DEEMPHASIZE : EMPHASIZE;
						DrwGame(false);
					}
				}
				break;
			}
		}
		break;

	case WM_KEYDOWN:
		{
			if( gameDlg ) break;

			bool move   = false;
			int  newRow = gameRow;
			int  newCol = gameCol;

			switch( wParam )
			{
			case _T('L'):   PostMessage(hWnd, WM_COMMAND, ID_GAME_LOAD,      0); break;
			case _T('S'):   PostMessage(hWnd, WM_COMMAND, ID_GAME_SAVE,      0); break;
			case _T('P'):   PostMessage(hWnd, WM_COMMAND, ID_GAME_PREV,      0); break;
			case _T('N'):   PostMessage(hWnd, WM_COMMAND, ID_GAME_NEXT,      0); break;
			case _T('D'):   PostMessage(hWnd, WM_COMMAND, ID_GAME_RANDOM,    0); break;
			case _T('U'):   PostMessage(hWnd, WM_COMMAND, ID_GAME_NUMBER,    0); break;
			case _T('R'):   PostMessage(hWnd, WM_COMMAND, ID_GAME_RESTART,   0); break;
			case VK_ESCAPE: PostMessage(hWnd, WM_COMMAND, ID_GAME_EXIT,      0); break;
			case _T('B'):   PostMessage(hWnd, WM_COMMAND, ID_TYPE_BEGINNER,  0); break;
			case _T('E'):   PostMessage(hWnd, WM_COMMAND, ID_TYPE_EASY,      0); break;
			case _T('M'):   PostMessage(hWnd, WM_COMMAND, ID_TYPE_MEDIUM,    0); break;
			case _T('H'):   PostMessage(hWnd, WM_COMMAND, ID_TYPE_HARD,      0); break;
			case _T('X'):   PostMessage(hWnd, WM_COMMAND, ID_TYPE_EXTREME,   0); break;
			case _T('A'):   PostMessage(hWnd, WM_COMMAND, ID_TYPE_ADJUST,    0); break;
			case _T('C'):   PostMessage(hWnd, WM_COMMAND, ID_SHOW_ERRORS,    0); break;
			case _T('V'):   PostMessage(hWnd, WM_COMMAND, ID_SHOW_POSSIBLES, 0); break;
			case VK_F1:     PostMessage(hWnd, WM_COMMAND, ID_GAME_HELP,      0); break;

			case _T(' '):   PostMessage(hWnd, WM_LBUTTONDBLCLK, 0, 0);           break;

			case VK_APPS:
			case VK_RETURN:
				{
					RECT tbr = Wnd_OuterRect(gameRBar);
					int  x   = gameCol*(colW + colM) + colM + colW/2;
					int  y   = gameRow*(rowH + rowM) + rowM + rowH/2+tbr.bottom-tbr.top;
					PostMessage(hWnd, WM_RBUTTONUP, 0, MAKELPARAM(x, y));
				}
				break;

			case VK_HOME:  move = true; newRow =  0;          newCol = 0;          break;
			case VK_END:   move = true; newRow =  SDSIZESQ-1; newCol = SDSIZESQ-1; break;
			case VK_LEFT:  move = true; newCol--; break;
			case VK_RIGHT: move = true; newCol++; break;
			case VK_UP:    move = true; newRow--; break;
			case VK_DOWN:  move = true; newRow++; break;
			case VK_TAB:
				{
					move = true;

					if( Sys_KeyIsDown(VK_SHIFT) )
					{
						newCol -= SDSIZE;

						if( newCol  < 0 )
						{
							newCol  = SDSIZESQ - (SDSIZE-gameCol-1) % SDSIZE - 1;
							newRow -= SDSIZE;

							if( newRow < 0 )
								newRow = SDSIZESQ - (SDSIZE-gameRow-1) % SDSIZE - 1;
						}
					}
					else
					{
						newCol += SDSIZE;

						if( newCol >= SDSIZESQ )
						{
							newCol  = gameCol % SDSIZE;
							newRow += SDSIZE;

							if( newRow >= SDSIZESQ )
								newRow  = gameRow % SDSIZE;
						}
					}
				}
				break;

			case _T('0'):
			case _T('1'):
			case _T('2'):
			case _T('3'):
			case _T('4'):
			case _T('5'):
			case _T('6'):
			case _T('7'):
			case _T('8'):
			case _T('9'):
				{
					if( !gameWon )
					{
						if( Sys_KeyIsDown(VK_SHIFT) )   gamePos = EMPHASIZE;
						if( Sys_KeyIsDown(VK_CONTROL) ) gamePos = DEEMPHASIZE;

						if( gamePos )
						{
							BYTE emp = gamePos;
							gamePos  = 0;
							if( wParam == _T('0') ) break;
							BYTE pos = wParam - _T('0') - 1;
							gamePoss[gameRow][gameCol][pos] =
							gamePoss[gameRow][gameCol][pos] == emp ? 0 : emp;
							DrwGame(false);
						}
						else
						{
							int nbr = wParam - _T('0');

							if( gameCell[gameRow][gameCol].fix == false &&
								gameCell[gameRow][gameCol].usr != nbr )
							{
								gameCell[gameRow][gameCol].usr  = nbr;
								gameCell[gameRow][gameCol].sho  = nbr > 0 ? true : false;
								gameWon                         = ChkGame(false);
								DrwGame(false);
							}
						}
					}
				}
				break;
			}

			if( move && ((newRow != gameRow) || (newCol != gameCol)) )
			{
				if( newCol < 0 )         { newRow--; newCol = SDSIZESQ-1; }
				if( newCol >= SDSIZESQ ) { newRow++; newCol = 0;          }

				if( newRow >= 0 && newRow < SDSIZESQ )
				{
					gameRow = newRow;
					gameCol = newCol;
					DrwGame(false);
				}
			}
		}
		break;

	case WM_LBUTTONDOWN:
	case WM_RBUTTONUP:
		{
			if( gameWon ) break;

			RECT   tbr =  Wnd_OuterRect(gameRBar);
			POINTS cp  =  MAKEPOINTS(lParam);
			int    tbh =  tbr.bottom - tbr.top;
			int    col =  cp.x     /(colW+colM);
			int    row = (cp.y-tbh)/(rowH+rowM);
			bool   sel =  col  != gameCol || row != gameRow;
			bool   lft =  uMsg == WM_LBUTTONDOWN;

			if( sel )
			{
				gameCol = col;
				gameRow = row;
				DrwGame(false);
			}

			if( gameDlg )
			{
				DestroyWindow(gameDlg);
				break;
			}

			if( (sel && lft) || gameCell[gameRow][gameCol].fix ) break;

			SetTimer(hWnd, DBLCLKCHK, lft ? GetDoubleClickTime() : 0, NULL);
		}
		break;

	case WM_TIMER:
		{
			if( wParam == TIMER )
			{
				StaGame(false);
				break;
			}

			if( wParam == DBLCLKCHK )
			{
				KillTimer(hWnd, DBLCLKCHK);
				DoPickNumber(hWnd);
			}
		}
		break;

	case WM_LBUTTONDBLCLK:
		{
			KillTimer(hWnd, DBLCLKCHK);

			if( gameDlg )
			{
				DestroyWindow(gameDlg);
				break;
			}

			if( !gameDlg && !gameWon )
			{
				int nbr = GetNextNumber(gameRow, gameCol);

				if( gameCell[gameRow][gameCol].fix == false &&
					gameCell[gameRow][gameCol].usr != nbr )
				{
					gameCell[gameRow][gameCol].usr  = nbr;
					gameCell[gameRow][gameCol].sho  = nbr > 0 ? true : false;
					gameWon                         = ChkGame(false);
					DrwGame(false);
				}
			}
		}
		break;

	case WM_MBUTTONDOWN:
	case WM_RBUTTONDOWN:
	case WM_XBUTTONDOWN:
	case WM_LBUTTONUP:
	case WM_MBUTTONUP:
	case WM_XBUTTONUP:
	case WM_NCLBUTTONDOWN:
	case WM_NCMBUTTONDOWN:
	case WM_NCRBUTTONDOWN:
	case WM_NCXBUTTONDOWN:
	case WM_NCLBUTTONUP:
	case WM_NCMBUTTONUP:
	case WM_NCRBUTTONUP:
	case WM_NCXBUTTONUP:
		{
			if( gameDlg )
			{
				DestroyWindow(gameDlg);
				break;
			}
		}
		break;

	case WM_MOUSEWHEEL:
		{
			return Sys_HandleMouseWheel(hWnd, wParam, lParam, false, true);
		}
		break;

	case WM_VSCROLL:
		{
			switch( wParam )
			{
			case SB_LINEDOWN: Sys_SendKeys(hWnd, _T("<d>")); break;
			case SB_LINEUP:   Sys_SendKeys(hWnd, _T("<u>")); break;
			}
		}
		break;

	case WM_PARENTNOTIFY:
		{
			if( gameDlg )
			{
				if( LOWORD(wParam) != WM_CREATE && LOWORD(wParam) != WM_DESTROY )
				{
					POINTS pts = MAKEPOINTS(lParam);
					POINT  pt  = { pts.x, pts.y };

					if( ChildWindowFromPoint(hWnd, pt) != gameDlg )
						DestroyWindow(gameDlg);
				}
			}
		}
		break;

	case WM_INITMENUPOPUP:
		{
			GameType  diffType[] = { gameBeginner,     gameEasy,     gameMedium,     gameHard,     gameExtreme     };
			UINT      diffItem[] = { ID_TYPE_BEGINNER, ID_TYPE_EASY, ID_TYPE_MEDIUM, ID_TYPE_HARD, ID_TYPE_EXTREME };
			const int n          =   SIZEOF(diffType);

			for( int i = 0; i < n; i++ )
				CheckMenuItem((HMENU)wParam, diffItem[i], MF_BYCOMMAND | (gameType == diffType[i] ? MF_CHECKED : MF_UNCHECKED));

			CheckMenuItem((HMENU)wParam, ID_SHOW_ERRORS,    MF_BYCOMMAND | (gameChk ? MF_CHECKED : MF_UNCHECKED));
			CheckMenuItem((HMENU)wParam, ID_SHOW_POSSIBLES, MF_BYCOMMAND | (gameSho ? MF_CHECKED : MF_UNCHECKED));
		}
		break;
	}

	ret = ret != -1 ? ret : DefWindowProc(hWnd, uMsg, wParam, lParam);

	return ret;
}

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
static void BegGame(bool draw, GameType type, bool force)
{
	if( !CanGame() ) return;

	if( force )
	{
		bool newGame = false;

		for( int row = 0; row < SDSIZESQ; row++ )
		{
			for( int col = 0; col < SDSIZESQ; col++ )
			{
				if( !gameCell[row][col].fix && gameCell[row][col].usr )
					newGame = true;

				for( int pos = 0; pos < SDSIZESQ; pos++ )
					gamePoss[row][col][pos] = 0; // go ahead and clear out possibles since we're already looping over row & col
			}
		}

		if( !newGame )
		{
			if( gameRow || gameCol )
			{
				gameRow =  gameCol = 0;
				DrwGame(true);
			}

			return;
		}
	}

	if( gameType != type || force )
	{
		gameType  = type;

		ExeGame(false);
	}
}

// ----------------------------------------------------------------------------
static void EndGame(bool draw)
{
	if( !gameWon )
	{
		EndGame();

		if( draw ) DrwGame(false);
	}
}

// ----------------------------------------------------------------------------
static void ExeGame(bool newGame)
{
	if( CanGame() )
	{
		DWORD  tid;
		HANDLE hThread = CreateThread(NULL, 0, ExeGameThread, (void*)(newGame ? 0 : 1), 0, &tid);
		CloseHandle(hThread);
	}
}

// ----------------------------------------------------------------------------
static DWORD WINAPI ExeGameThread(void* lParam)
{
	gameExe = true;

	SetThreadPriority(GetCurrentThread(), THREAD_PRIORITY_LOWEST);

	for( int row = 0; row < SDSIZESQ; row++ )
	{
		for( int col = 0; col < SDSIZESQ; col++ )
		{
			for( int pos = 0; pos < SDSIZESQ; pos++ )
				gamePoss[row][col][pos] = 0;
		}
	}

	switch( (int)lParam )
	{
	case 0: NewGame(); break;
	case 1: BegGame(); break;
	}

	gameChk = !gameChk;
	VerGame(true, true);

	gameExe = false;

	return 0;
}

// ----------------------------------------------------------------------------
static void GetGameBoardBmap(int colW, int rowH, int wndW, int wndH)
{
	HDC      hDC     = GetDC(NULL);
	HDC      hSC     = CreateCompatibleDC(hDC);
	HDC      hBC     = CreateCompatibleDC(hDC);
	HGDIOBJ  hSCBmp  = GetCurrentObject  (hSC, OBJ_BITMAP);
	HGDIOBJ  hBCBmp  = GetCurrentObject  (hBC, OBJ_BITMAP);
	int      hSCsbm  = SetStretchBltMode (hSC, COLORONCOLOR);
	int      hBCsbm  = SetStretchBltMode (hBC, COLORONCOLOR);

	HBITMAP* games[] = { &gameEmpty, &gameFixed, &gameHlite, &gameSolve, &gameCheck, &gameError };
	HBITMAP* draws[] = { &drawEmpty, &drawFixed, &drawHlite, &drawSolve, &drawCheck, &drawError };
	SIZE     bmpsz;

	for( int b = 0; b < SIZEOF(games); b++ )
	{
		if( *draws[b] ) DeleteObject(*draws[b]);
		if( *games[b] )
		{
			*draws[b] = CreateCompatibleBitmap(hDC, colW, rowH);
			bmpsz     = Sys_BitmapSize(*games[b]);
			SelectObject(hSC, *games[b]);
			SelectObject(hBC, *draws[b]);
			StretchBlt  (hBC, 0, 0, colW, rowH, hSC, 0, 0, bmpsz.cx, bmpsz.cy, SRCCOPY);
		}
	}

	SetStretchBltMode(hSC, HALFTONE);
	SetStretchBltMode(hBC, HALFTONE);

	for( int n = 0; n < 9; n++ )
	{
		if( drawNumbr[n] ) DeleteObject(drawNumbr[n]);
		if( gameNumbr[n] )
		{
			drawNumbr[n] = CreateCompatibleBitmap(hDC, colW, rowH);
			bmpsz        = Sys_BitmapSize(gameNumbr[n]);
			SelectObject(hSC, gameNumbr[n]);
			SelectObject(hBC, drawNumbr[n]);
			StretchBlt  (hBC, 0, 0, colW, rowH, hSC, 0, 0, bmpsz.cx, bmpsz.cy, SRCCOPY);
		}
	}

	if( gameBkgnd )
	{
		if( drawBkgnd ) DeleteObject(drawBkgnd);
		if( gameBkgnd )
		{
			drawBkgnd = CreateCompatibleBitmap(hDC, wndW, wndH);
			bmpsz     = Sys_BitmapSize(gameBkgnd);
			SelectObject(hSC, gameBkgnd);
			SelectObject(hBC, drawBkgnd);
			StretchBlt  (hBC, 0, 0, wndW, wndH, hSC, 0, 0, bmpsz.cx, bmpsz.cy, SRCCOPY);
		}
	}

	SetStretchBltMode(hSC, hSCsbm);
	SetStretchBltMode(hBC, hBCsbm);
	SelectObject(hSC, hSCBmp);
	SelectObject(hBC, hBCBmp);
	ReleaseDC(NULL, hDC);
	DeleteDC(hBC);
	DeleteDC(hSC);
}

// ----------------------------------------------------------------------------
static void GetGameBoardFont(int rowH, HFONT& hFont1, HFONT& hFont2)
{
	static int pps1 = 0, pps2 = 0;

	int ps1 = max(6, rowH*28/49); // 49 is the initial size I designed against
	int ps2 = max(6, rowH* 8/49);

	if( ps1 != pps1 || ps2 != pps2 )
	{
		LOGFONT lf;
		HDC     hDC = GetDC(NULL);
		int     dpi = GetDeviceCaps(hDC, LOGPIXELSY);
		int     ok1 = ReleaseDC(NULL, hDC);

		if( pps1 != ps1 )
		{
			pps1  = ps1;
			DeleteObject(hFont1);
			memcpy(&lf, &gameLogF1, sizeof(lf));
			lf.lfHeight = -MulDiv(ps1, dpi, 72);
			hFont1      =  CreateFontIndirect(&lf);
		}

		if( pps2 != ps2 )
		{
			pps2  = ps2;
			DeleteObject(hFont2);
			memcpy(&lf, &gameLogF2, sizeof(lf));
			lf.lfHeight = -MulDiv(ps2, dpi, 72);
			hFont2      =  CreateFontIndirect(&lf);
		}
	}
}

// ----------------------------------------------------------------------------
static void GetGameBoardSize(const RECT& cr, int& colM, int& rowM, int& subM, int& colW, int& rowH, int& colH, int& rowW)
{
	int colm = 2;
	int rowm = 2;
	int subm = SDMARGIN;
	int roww = cr.right  - cr.left;
	int colh = cr.bottom - cr.top;
	int colw = (roww - (SDSIZESQ+1-(SDSIZE-1))*colm - (SDSIZE-1)*subm)/SDSIZESQ;
	int rowh = (colh - (SDSIZESQ+1-(SDSIZE-1))*rowm - (SDSIZE-1)*subm)/SDSIZESQ;

	colh     = SDSIZESQ*rowh + (SDSIZESQ+1-(SDSIZE-1))*rowm + (SDSIZE-1)*subm;
	roww     = SDSIZESQ*colw + (SDSIZESQ+1-(SDSIZE-1))*colm + (SDSIZE-1)*subm;
	subm     = subm  - colm;

	colM     = colm;
	rowM     = rowm;
	subM     = subm;
	colW     = colw;
	rowH     = rowh;
	colH     = colh;
	rowW     = roww;
}

// ----------------------------------------------------------------------------
static void LodGame()
{
	HBITMAP* hBmps[] = {    &gameEmpty,          &gameFixed,          &gameHlite,          &gameSolve,          &gameCheck,          &gameError,          &gameBkgnd       };
	LPCTSTR  bmpfs[] = { _T("cellempty.bmp"), _T("cellfixed.bmp"), _T("cellhlite.bmp"), _T("cellsolve.bmp"), _T("cellcheck.bmp"), _T("cellerror.bmp"), _T("backfixed.bmp") };
	UINT     bmpid[] = { IDB_CELLEMPTY,       IDB_CELLFIXED,       IDB_CELLHLITE,       IDB_CELLSOLVE,       IDB_CELLCHECK,       IDB_CELLERROR,       0                   };

	for( int b = 0; b < SIZEOF(hBmps); b++ )
	{
		*hBmps[b] = (HBITMAP)LoadImage(NULL, bmpfs[b], IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE);

		if( !(*hBmps[b]) && bmpid[b] )
		*hBmps[b] = (HBITMAP)LoadImage(GetModuleHandle(NULL), MAKEINTRESOURCE(bmpid[b]), IMAGE_BITMAP, 0, 0, LR_DEFAULTCOLOR);
	}

	for( int n = 0; n < 9; n++ )
	{
		TCHAR file[] = _T("0.bmp");
		file[0]      = _T('1') + n;
		gameNumbr[n] = (HBITMAP)LoadImage(NULL, file, IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE);
	}
}

// ----------------------------------------------------------------------------
static void NxtGame(bool draw)
{
	if( CanGame() ) NewGame(draw);
}

// ----------------------------------------------------------------------------
static void PrvGame(bool draw)
{
	if( gameSeed > 1 && CanGame() )
	{
		gameSeed -= 2; // adjust by 2 since Sudoku::NewGame will bump the value
		NewGame(draw);
	}
}

// ----------------------------------------------------------------------------
static void ShoGame(bool draw)
{
	if( !gameWon )
	{
		gameSho = !gameSho;
		gameWon = ChkGame(false);

		Toolbar_BtnCheck(gameTBar, ID_SHOW_POSSIBLES, gameSho);

		if( draw ) DrwGame(false);
	}
}

// ----------------------------------------------------------------------------
static void TimGame(TCHAR* ctime, int ltime, bool timeGame, bool first)
{
	static LARGE_INTEGER gameStart;
	static LARGE_INTEGER gameEnd;

	if( first )
	{
		FILETIME                 ft;
		GetSystemTimeAsFileTime(&ft);

		gameStart.LowPart  = ft.dwLowDateTime;
		gameStart.HighPart = ft.dwHighDateTime;
		gameEnd.QuadPart   = 0;
	}

	SYSTEMTIME    st;
	GetLocalTime(&st);

	int msOffset = st.wMilliseconds % TIMERRES;

	if( msOffset )
	{
		KillTimer(gameWnd, TIMER);
		SetTimer (gameWnd, TIMER, TIMERRES-msOffset, NULL);
	}

	if( timeGame )
	{
		FILETIME                 lt,  ft;
		SystemTimeToFileTime   (&st, &lt);
		LocalFileTimeToFileTime(&lt, &ft);

		if( gameWon == false )
		{
			gameEnd.LowPart  = ft.dwLowDateTime;
			gameEnd.HighPart = ft.dwHighDateTime;
			gameEnd.QuadPart = first ? 0 : gameEnd.QuadPart - gameStart.QuadPart;
		}

		ft.dwLowDateTime  = gameEnd.LowPart;
		ft.dwHighDateTime = gameEnd.HighPart;

		TCHAR sep[32];
		FileTimeToSystemTime(&ft, &st);
		GetLocaleInfo(LOCALE_USER_DEFAULT, LOCALE_STIME, sep, SIZEOF(sep));
		_sntprintf(ctime, ltime-1, _T("%2.2d%s%2.2d%s%2.2d"), st.wHour, sep, st.wMinute, sep, st.wSecond);
	}
	else
		GetTimeFormat(LOCALE_USER_DEFAULT, 0, &st, NULL, ctime, ltime);
}

// ----------------------------------------------------------------------------
static void XitGame()
{
	HGDIOBJ* objs[] =
	{
		(HGDIOBJ*)&gameFont1, (HGDIOBJ*)&gameFont2,
		(HGDIOBJ*)&gameEmpty, (HGDIOBJ*)&gameFixed, (HGDIOBJ*)&gameHlite, (HGDIOBJ*)&gameSolve, (HGDIOBJ*)&gameCheck, (HGDIOBJ*)&gameError, (HGDIOBJ*)&gameBkgnd,
		(HGDIOBJ*)&drawEmpty, (HGDIOBJ*)&drawFixed, (HGDIOBJ*)&drawHlite, (HGDIOBJ*)&drawSolve, (HGDIOBJ*)&drawCheck, (HGDIOBJ*)&drawError, (HGDIOBJ*)&drawBkgnd
	};

	for( int b = 0; b < SIZEOF(objs); b++ )
		DeleteObject(*objs[b]);
}

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
bool CanGame()
{
	return gameGen == false && gameExe == NULL;
}

// ----------------------------------------------------------------------------
void DrwGame(bool start, HDC hWC)
{
	RECT     r;
	RECT     cr      = Wnd_InnerRect(gameWnd);
	RECT     tbr     = Wnd_OuterRect(gameRBar);
	RECT     sbr     = Wnd_OuterRect(gameSBar);

	int      tbh     = tbr.bottom - tbr.top;
	int      sbh     = sbr.bottom - sbr.top;
	cr.top           = tbh;
	cr.bottom       -= sbh;

	RECT     br      = { 0, 0, cr.right - cr.left, cr.bottom - cr.top };

	HDC      hDC     = hWC == NULL ? GetDC(gameWnd) : hWC;
	HDC      hBC     = CreateCompatibleDC(hDC);
	HDC      hSC     = CreateCompatibleDC(hDC);

	HBITMAP  hBitmap = CreateCompatibleBitmap(hDC, br.right, br.bottom);

	HGDIOBJ  hBCFnt  = SelectObject(hBC, gameFont1);
	HGDIOBJ  hBCBmp  = SelectObject(hBC, hBitmap);
	HGDIOBJ  hSCBmp  = GetCurrentObject(hSC, OBJ_BITMAP);

	COLORREF edge    = RGB(0, 0, 0);
	COLORREF back    = gameWon ? RGB(0, 192, 0) : RGB(192, 192, 192);
	COLORREF enter   = gameWon ? RGB(0, 192, 0) : RGB(224, 224, 224);
	COLORREF error   = gameWon ? RGB(0, 192, 0) : RGB(255, 0,   0);
	COLORREF group   = RGB(0, 0, 0);
	COLORREF text1   = gameClrF1;
	COLORREF text2   = gameClrF2;

	HBRUSH   hEdge   = CreateSolidBrush(edge);
	HBRUSH   hErase  = CreateSolidBrush(RGB(208, 208, 208));
	HBRUSH   hBack   = CreateSolidBrush(back);
	HBRUSH   hEnter  = CreateSolidBrush(enter);
	HBRUSH   hBadNbr = CreateSolidBrush(error);
	HBRUSH   hGroup  = CreateSolidBrush(group);

	FillRect (hBC, &br, hErase);
	FrameRect(hBC, &br, hEdge);

	int oldW = colW;
	int oldH = rowH;
	GetGameBoardSize(cr, colM, rowM, subM, colW, rowH, colH, rowW);
	if( colW != oldW || rowH != oldH )
		GetGameBoardBmap(colW, rowH, cr.right-cr.left, cr.bottom-cr.top);

	SetBkMode (hBC, TRANSPARENT);
	SetBkColor(hBC, back);

	for( int row = 0; row < SDSIZESQ; row++ )
	{
		for( int col = 0; col < SDSIZESQ; col++ )
		{
			int  x   =  col*(colW + colM) + (col/SDSIZE)*subM + colM;
			int  y   =  row*(rowH + rowM) + (row/SDSIZE)*subM + rowM;
			bool sel = (row == gameRow) && (col == gameCol);
			bool fix =  gameCell[row][col].fix;
			bool mty =  gameCell[row][col].usr  == 0;

			SetRect(&r, x, y, x+colW, y+rowH);

			if( gameWon )
			{
				if( drawBkgnd )
				{
					SelectObject(hSC, drawBkgnd);
					BitBlt(hBC, r.left, r.top, r.right-r.left, r.bottom-r.top, hSC, r.left, r.top, SRCCOPY);
				}
				else
				if( drawSolve )
				{
					SelectObject(hSC, drawSolve);
					BitBlt(hBC, r.left, r.top, r.right-r.left, r.bottom-r.top, hSC, 0, 0, SRCCOPY);
				}
				else
					FillRect(hBC, &r, fix ? hBack : hEnter);
			}
			else if( gameChk && gameErrs[row][col] )
			{
				if( mty )
				{
					if( drawCheck )
					{
						SelectObject(hSC, drawCheck);
						BitBlt(hBC, r.left, r.top, r.right-r.left, r.bottom-r.top, hSC, 0, 0, SRCCOPY);
					}
					else
						FillRect(hBC, &r, hBadNbr);
				}
				else
				{
					if( drawError )
					{
						SelectObject(hSC, drawError);
						BitBlt(hBC, r.left, r.top, r.right-r.left, r.bottom-r.top, hSC, 0, 0, SRCCOPY);
					}
					else
						FillRect(hBC, &r, hBadNbr);
				}
			}
			else
			{
				if( fix )
				{
					if( drawBkgnd )
					{
						SelectObject(hSC, drawBkgnd);
						BitBlt(hBC, r.left, r.top, r.right-r.left, r.bottom-r.top, hSC, r.left, r.top, SRCCOPY);
					}
					else
					if( drawFixed )
					{
						SelectObject(hSC, drawFixed);
						BitBlt(hBC, r.left, r.top, r.right-r.left, r.bottom-r.top, hSC, 0, 0, SRCCOPY);
					}
					else
					{
						FillRect(hBC, &r, hBack);
					}
				}
				else
				{
					if( drawBkgnd && gameCell[row][col].usr == gameCell[row][col].nbr )
					{
						SelectObject(hSC, drawBkgnd);
						BitBlt(hBC, r.left, r.top, r.right-r.left, r.bottom-r.top, hSC, r.left, r.top, SRCCOPY);
					}
					else
					if( drawEmpty )
					{
						SelectObject(hSC, drawEmpty);
						BitBlt(hBC, r.left, r.top, r.right-r.left, r.bottom-r.top, hSC, 0, 0, SRCCOPY);
					}
					else
					{
						FillRect(hBC, &r, hEnter);
					}
				}
			}

			if( sel )
			{
				if( drawHlite )
				{
					SelectObject(hSC, drawHlite);
					BitBlt(hBC, r.left, r.top, r.right-r.left, r.bottom-r.top, hSC, 0, 0, SRCCOPY);
				}
				else
				{
					HBRUSH hSelect = GetSysColorBrush(COLOR_BTNHILIGHT);
					InflateRect(&r, -colM*2, -rowM*2);
					FillRect(hBC, &r, hSelect);
					InflateRect(&r,  colM*2,  rowM*2);
				}
			}

			if( gameCell[row][col].sho )
			{
				int idx = gameCell[row][col].usr ? gameCell[row][col].usr : gameCell[row][col].nbr;

				if( drawNumbr[idx-1] )
				{
					SelectObject(hSC, drawNumbr[idx-1]);
					BitBlt(hBC, r.left, r.top, r.right-r.left, r.bottom-r.top, hSC, 0, 0, SRCCOPY);
				}
				else
				{
					TCHAR nbr[] = _T("0");
					nbr[0]      = gameMapF1[idx-1];
					SetTextColor(hBC, text1);
					DrawText(hBC, nbr, -1, &r, DT_CENTER | DT_SINGLELINE | DT_VCENTER);
				}
			}
			else
			{
				int   poss;
				TCHAR nbr[] = _T("0");
				bool  possible[SDSIZESQ];

				SelectObject(hBC, gameFont2);
				r.left   += 2;
				r.top    += 3;
				r.right  -= 2;
				r.bottom -= 2;

				int  fro =   0;
				int  fco =   0;
				int  fdw =  (r.right -r.left)/3;
				int  fdh =  (r.bottom-r.top) /3;
				RECT fr  = { r.left,  r.top, r.left + fdw, r.top + fdh };

				const UINT flags[] =
				{
					DT_RIGHT | DT_BOTTOM,  DT_CENTER | DT_BOTTOM,  DT_LEFT | DT_BOTTOM,
					DT_RIGHT | DT_VCENTER, DT_CENTER | DT_VCENTER, DT_LEFT | DT_VCENTER,
					DT_RIGHT | DT_TOP,     DT_CENTER | DT_TOP,     DT_LEFT | DT_TOP
				};

				GetPossibles(row, col, possible);

				for( int n = 1; n <= SDSIZESQ; n++ )
				{
					poss = gamePoss[row][col][n-1];

					if( poss || (gameSho && possible[n-1]) )
					{
						nbr[0] = gameMapF2[n-1];

						if( poss )
						{
							bool     demp  = poss == DEEMPHASIZE;
							BYTE     red   = min(255, max(0, 222 + (demp ? -32 :  32)));
							BYTE     green = min(255, max(0, 231 + (demp ? -32 : -96)));
							BYTE     blue  = min(255, max(0, 239 + (demp ? -32 : -96)));
							COLORREF color = RGB(red, green, blue);
							SetTextColor(hBC, color);
						}

						SetTextColor(hBC, text2);
						DrawText(hBC, nbr, -1, &fr, flags[n-1] | DT_SINGLELINE);
						SetTextColor(hBC, text1);
					}

					switch( n )
					{
					case 3:
					case 6:  fr.left    = r.left;
							 fr.right   = r.left + fdw;
							 fr.top    += fdh;
							 fr.bottom += fdh;
							 break;
					default: fr.left   += fdw;
							 fr.right  += fdw;
							 break;
					}
				}

				SelectObject(hBC, gameFont1);
			}
		}
	}

	{
		int ox, oy;
		ox = SDSIZE*1*(colW+colM)+0*subM;
		oy = SDSIZE*1*(rowH+rowM)+0*subM;
		SetRect(&r, ox, 0,  ox+subM+colM, cr.bottom);    FillRect(hBC, &r, hGroup);
		SetRect(&r, 0,  oy, cr.right,     oy+subM+rowM); FillRect(hBC, &r, hGroup);
		ox = SDSIZE*2*(colW+colM)+1*subM;
		oy = SDSIZE*2*(rowH+rowM)+1*subM;
		SetRect(&r, ox, 0,  ox+subM+colM, cr.bottom);    FillRect(hBC, &r, hGroup);
		SetRect(&r, 0,  oy, cr.right,     oy+subM+rowM); FillRect(hBC, &r, hGroup);
	}

	DeleteObject(hEdge);
	DeleteObject(hErase);
	DeleteObject(hBack);
	DeleteObject(hEnter);
	DeleteObject(hBadNbr);
	DeleteObject(hGroup);

	BitBlt(hDC, cr.left, cr.top, cr.right-cr.left, cr.bottom-cr.top, hBC, 0, 0, SRCCOPY);

	SelectObject(hBC, hBCFnt);
	SelectObject(hBC, hBCBmp);
	SelectObject(hSC, hSCBmp);
	DeleteObject(hBitmap);
	DeleteDC    (hBC);
	DeleteDC    (hSC);

	if( hWC == NULL )
		ReleaseDC(gameWnd, hDC);

	Sys_ResetMemUsage();

	StaGame(start);
}

// ----------------------------------------------------------------------------
void NewGame(bool draw)
{
	if( CanGame() ) ExeGame(true);
}

// ----------------------------------------------------------------------------
void RndGame(bool draw)
{
	if( CanGame() )
	{
		gameSeed = (int)((float)rand()/(float)RAND_MAX*MAXGAME);
		NewGame(draw);
	}
}

// ----------------------------------------------------------------------------
void StaGame(bool start)
{
	TCHAR   ctext[128], cleft[128], ctime[128];
	int     textw,      leftw,      timew;

	LPCTSTR sect    =   _T("Statusbar");
	LPCTSTR types[] = { _T("Beginner"), _T("Easy"), _T("Medium"), _T("Hard"), _T("Extreme") };
	LPCTSTR type    =   LCL_String(sect, types[gameType-gameBeginner]);
	int     maxdif  =   gameDiff[gameType-gameMinimum];

	if( gameGen )
	{
		_sntprintf(ctext, SIZEOF(ctext)-1, LCL_String(sect, _T("%s (%d%% max) puzzle %d")),
				   type, maxdif, gameSeed);
		_sntprintf(cleft, SIZEOF(cleft)-1, LCL_String(sect, _T("Generating (%d solutions analyzed)...")),
				   gameSolves);
	}
	else
	{
		_sntprintf(ctext, SIZEOF(ctext)-1, LCL_String(sect, _T("%s (%d%% max) puzzle %d (rated at %d%%)")),
				   type, maxdif, gameSeed, gameLevel);

		int  gameDone = 0;
		for( int row  = 0; row < SDSIZESQ; row++ )
		{
			for( int col = 0; col < SDSIZESQ; col++ )
			{
				if( gameCell[row][col].sho ) gameDone++;
			}
		}

		_sntprintf(cleft, SIZEOF(cleft)-1, LCL_String(sect, _T("%d cells done (%.1f%%) %d to go")),
				   gameDone, (float)gameDone*100.0f/(float)(SDSIZESQ*SDSIZESQ),
				   SDSIZESQ*SDSIZESQ-gameDone);
	}

	TimGame(ctime, SIZEOF(ctime), gameTim, start);

	RECT    tr;
	HDC     hDC    = GetDC(gameSBar);
	HGDIOBJ hDCFnt = SelectObject(hDC, (HFONT)SendMessage(gameSBar, WM_GETFONT, 0, 0));

	SetRect (&tr, 0, 0, 0, 0);
	DrawText(hDC, cleft, -1, &tr, DT_CALCRECT | DT_SINGLELINE);
	leftw = tr.right - tr.left + 16; // 16 for padding

	SetRect (&tr, 0, 0, 0, 0);
	DrawText(hDC, ctime, -1, &tr, DT_CALCRECT | DT_SINGLELINE);
	timew = tr.right - tr.left + 24; // 24 for padding

	int pos[] = { leftw, timew, 0 };
	Statusbar_SetParts(gameSBar, pos, SIZEOF(pos), true);

	SetRect (&tr, 0, 0, 0, 0);
	DrawText(hDC, ctext, -1, &tr, DT_CALCRECT | DT_SINGLELINE);
	textw = tr.right - tr.left;

	if( textw > pos[0] )
	{
		_sntprintf(ctext, SIZEOF(ctext)-1, LCL_String(sect, _T("%s puzzle %d (rated at %d%%)")),
				   type, gameSeed, gameLevel);

		SetRect (&tr, 0, 0, 0, 0);
		DrawText(hDC, ctext, -1, &tr, DT_CALCRECT | DT_SINGLELINE);
		textw = tr.right - tr.left;

		if( textw > pos[0] )
			_sntprintf(ctext, SIZEOF(ctext)-1, LCL_String(sect, _T("%s puzzle %d")), type, gameSeed);
	}

	SelectObject(hDC, hDCFnt);
	ReleaseDC(gameSBar, hDC);

	LPCTSTR texts[] = { ctext, cleft, ctime };
	Statusbar_Update(gameSBar, texts, SIZEOF(texts));
}

// ----------------------------------------------------------------------------
void VerGame(bool draw, bool start)
{
	if( !gameWon )
	{
		gameChk = !gameChk;

		Toolbar_BtnCheck(gameTBar, ID_SHOW_ERRORS, gameChk);

		if( gameChk )
			gameWon = ChkGame(false);

		if( draw ) DrwGame(start);
	}
}
